/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({
  is: 'maximo-inspector-start',

  behaviors: [BaseComponent, Inspections, ResizeBehavior],

  listeners: {
	  'inspections-screen-resize' : '_screenResize'

  },

  properties: {

    /**
     * List of inspection form
     */
    dataSet: {
      type: Array,
      value: []
    },

    pendingCreation: {
      type: Array,
      value: []
    },

    /**
     * Processed list based on data set
     * Merge of dataSet and pendingCreation
     */
    instructionList: {
      type: Array,
      value: []
    },

    /**
     * 
     */
    selectedTile: {
      type: Object    	
    },

    /**
     * Computed label displayed in the header of the page 
     */
    headerTitle: {
      type: String,
      computed: '_computeHeaderTitle(totalInspResults)'
    },

    /**
     * Total forms results
     * pending creation + created
     */
    totalInspResults: {
      type: Number,
      value: 0
    },

    /**
     * flag to indicate creation of inspection result
     * or just status update
     */
    isResultCreated: {
      type: Boolean,
      value: true,
      readOnly: true
    },

  },

  observers: ['mergeLists(dataSet, pendingCreation)'],

  /**
   * Observer complex dataSet + pendingCreation
   */
  mergeLists: function(dataSet, pendingCreation) {
    var inspResults = 0;
    if (dataSet.length || pendingCreation.length) {
      var instructionList = this._computeInstructionList();
      this.set('instructionList', instructionList);
      inspResults = dataSet.length + pendingCreation.length;
    }
    
    this.totalInspResults = inspResults;
  },

  /**
   * Computes property headerTitle when dataSet changes 
   */
  _computeHeaderTitle: function(totalInsp) {

    var length = totalInsp;

    if (length === 1) {
      var list = (length === this.pendingCreation.length) ? this.pendingCreation : this.dataSet;
      var inspForm = list[0].inspectionform[0];
      return inspForm.name;
    } else {
      return $M.localize('uitext','mxapiinspresult','batched_inspection',[length]);
    }
  },

  /**
   * Group instructions of inspection results based 
   * on inspection form and apply sort
   * @return Array
   */
  _computeInstructionList: function() {

    var instructionList = [];
    var existingIndex;
    var obj;

    for (var inspResult of this.dataSet) {
      if (inspResult.status_maxvalue === 'PENDING' || !inspResult.href) {
        // Check if formid exists in list

        existingIndex = this.getFormIndex(inspResult.inspectionform[0], instructionList);
        if (existingIndex >= 0) {
          // Add ref object in list at existingIndex
          this.addReferenceObject(inspResult, instructionList[existingIndex].typeList);
        } else {
          obj = this.buildInstructionObject(inspResult);
          instructionList.push(obj);
        }
      }
    }

    for (var inspResult of this.pendingCreation) {
      if (inspResult.status_maxvalue === 'PENDING' || !inspResult.href) {
        // Check if formid exists in list

        existingIndex = this.getFormIndex(inspResult.inspectionform[0], instructionList);
        if (existingIndex >= 0) {
          // Add ref object in list at existingIndex
          this.addReferenceObject(inspResult, instructionList[existingIndex].typeList);
        } else {
          obj = this.buildInstructionObject(inspResult);
          instructionList.push(obj);
        }
      }
    }

    if (this.pendingCreation.length) {
      this._setIsResultCreated(false);
    }

    // Alphabetical sort for the Forms
    instructionList.sort(function (a, b) {
      if (a.formName > b.formName) {
        return 1;
      }
      if (a.formName < b.formName) {
        return -1;
      }
      // a must be equal to b
      return 0;
    });

    // Alphabetical sort also for Assets/ Locations
    var typeList, items =  [];
    instructionList.forEach(function(formInstruction) {
      typeList = formInstruction.typeList;
      typeList.forEach(function(itemList) {
        itemList.list.sort();
      });
    });


    return instructionList;
  },

  /**
   * extract form from list
   */
  getFormIndex: function(form, list) {
    var index = -1;
    for (var i = 0; i < list.length; i++) {
      if (form.inspectionformid === list[i].formId) {
        index = i;
        break;
      }
    }
    return index;
  },

  /**
   * Extract required info from inspresult to new object
   */
  buildInstructionObject: function(inspResult) {
    var elem = {};

    // All reference types
    var refArr = [];
    this.addReferenceObject(inspResult, refArr);
    elem.typeList = refArr;

    // Hide WO info
    //elem.hideWOInfo = this._computedHideWOIcon(inspResult);
    // ToolTip obj
    //elem.tooltipObject = this._computedTooltipObject(inspResult);

    var inspForm = inspResult.inspectionform[0];
    // Form name
    elem.formName = inspForm.name;
    // Form id
    elem.formId = inspForm.inspectionformid;
    // Has Instructions
    elem.hasInstructions = this._checkInstruction(inspForm);
    // Instructions
    elem.instructions = inspForm.description_longdescription;

    return elem;
  },

  /**
   * Opens Create Inspection View in order to start the inspection form creation.
   */
  createUnscheduledInspection: function(e) {
    this.fire('createUnscheduledInspection');
  },

  /**
   * Apply flag based on long description field
   */
  _checkInstruction: function(inspForm) {

    return ((!inspForm) || 
        (inspForm.description_longdescription === null) || 
        (inspForm.description_longdescription===undefined) ) ? false : true;
  },

  /**
   * Check instruction length
   * @return Boolean
   */
  isBatch: function () {
    return (this.totalInspResults > 1) ? true : false; 
  },
  
  /**
   * Clear lists
   */
  reset: function() {
    this.set('dataSet', []);
    this.set('pendingCreation', []);
  },

  /**
   * DEPRECATED
   * Trigger process that merge created forms with pending ones
   * and generates intruction list
   */
  processInstructionList: function() {
    var instructionList = this._computeInstructionList();
    this.set('instructionList', instructionList);
  },
  
  /**
   * Listen to screen resize
   * @param {*} ev 
   */
  _screenResize: function(ev) {

    ev.stopPropagation();

    const isSmallDevice = this.deviceType === deviceTypes_.PHONE 
      || (this.deviceType === deviceTypes_.TABLET 
        && this.orientation === orientationTypes_.PORTRAIT);

    if (isSmallDevice){
    	$j(this.$.inspectorListBckgroundImgDiv).toggleClass('incompleteimageMobile',true);
    } else {
    	$j(this.$.inspectorListBckgroundImgDiv).toggleClass('incompleteimageMobile',false);
    }

  }

    
});
