/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({
  is: 'maximo-inspector-tile',

  behaviors: [BaseComponent,Inspections,Select],

  listeners: {
	'long-touch'		: 'longTouchHandler',
    'selector-shown'	: 'selectorShown',
    'selector-hidden'	: 'selectorHidden'
  },

  properties: {

    /**
     * inspection form record
     */
    record: {
      type: Object,
      observer: '_objChanged'
    },

    /**
     * Revision number
     */
    _revision: {
      type: String
    },

    /**
     * Synonym domain of 
     * current status
     */
    state: {
      type: Object,
      notify: true
    },

    /**
     * Translation of current status
     */
    booleanState: {
      type: Boolean,
      observer: '_boolStateChange'
    },

    /**
     * Synonym domain collection
     */
    formStatusSet: {
      type: Array,
      observer: '_domainStatusChange'
    },

    /**
     * Flag to hide revision option
     */
    hideRevision: {
      type: Boolean,
      value: false
    },

    /**
     * Flag to hide delete option
     */
    hideDelete: {
      type: Boolean,
      value: false
    },

    /**
     * Flag to hide edit option
     */
    hideEdit: {
      type: Boolean,
      value: false
    },

    /**
     * Interval in seconds to 
     * undo delete operation
     */
    undoIntervalSecs: {
      Type: Number,
      value: 7,
      readOnly: true
    },

    /**
     * compute icon of object reference
     */
    referenceIcon: {
      type: String,
      computed: '_computedIcon(record)'
    },

    /**
     * compute info of object reference
     */
    referenceInfo: {
      type: String,
      computed: '_computedInfo(record)'
    },

    tooltipObject: {
      type: Object,
      computed: '_computedTooltipObject(record)'
    },
    
    tileListName: {
      type: Object,
      computed: '_computedTileListName(record)'
    },

    /**
     * Flag to hide wo tooptip icon
     */
    hideWOInfo: {
      type: Boolean,
      computed: '_computedHideWOIcon(record)'
    },

    filter: {
      type: String,
      observer: '_filterChanged'
    },

    /**
     * Complete Date attribute computed
     */
    completedDate: {
      type: String,
      computed: '_computedCompletedDate(record)'
    },

    /**
     * Ellipsis size of description label
     */
    ellipsis: {
      type: Number,
      value: 50,
      readOnly: true
    },

    /**
     * Flag to apply stacked cards style
     */
    isStacked: {
      type: Boolean,
      value: false,
      reflectToAttribute: true,
      computed: '_computedIsStacked(record)'
    },

    /**
     * List displayed in stacked tooltip
     */
    childrenInfoList: {
      type: Array,
      value: []
    },

    /**
     * Flag to avoid race condition with long touch
     */
    tapStopper: {
    	type: Boolean,
    	value: false
    },

    /**
     * DEPRECATED
     * Flag to hide Asset Description
     */
    hideAsset: {
      type: Boolean,
      value: true
    }, 

    /**
     * DEPRECATED
     * Flag to hide Location Description
     */
    hideLocation: {
      type: Boolean,
      value: true
    },

    /**
     * DEPRECATED
     * Current Page
     */
    page: {
      type: String
    },

    /**
     * DEPRECATED
     * Flag to hide Result List Tile
     */
    hideResultTile: {
      type: Boolean,
      value: true
    },

    /**
     * DEPRECATED
     * Flag to hide Create Tile
     */
    hideCreateTile: {
      type: Boolean,
      value: true
    },

  },

  ready : function(e){ },

  /**
   * Add listener
   */
  attached: function() {
    this.setSelectHostComponent();
    this.listen(this, 'tap', 'checkTap');
  },

  /**
   * Remove listerner
   */
  detached: function() {
    this.unlisten(this, 'tap', 'checkTap');
  },

  /**
   * Prevent card details when selector is tapped
   * Fires different event to open card details page 
   */
  checkTap: function(e) {
    e.stopPropagation();

    if (this.tapStopper) {
    	this.set('tapStopper', false);
    	return;
    }

    //during tap out, event target lost parent reference
    var cb = e.target.closest(this.selectorComponent);
    if (cb) {
      e.preventDefault();
      return;
    }

    //if tap belongs to selectorContainer
    //var checkboxId = e.target.id || e.target.parentElement.id
    //var node = this._selectorContainer.querySelector('#'+checkboxId);
    this.fire('card-tap',e);
  },

  /**
   * Set where the selector sits in card
   */
  setSelectHostComponent: function() {
    this._selectorContainer = this.$.selectHost;
  },

  _setUndoIntervalSecs: function(secs){
    this.undoIntervalSecs = secs;
  },

  /**
   * Observer record
   */
  _objChanged: function (newVal) {
    //this.fillTooltip(newVal);
    this.fillFormOptions(newVal);
    this.mountRevision(newVal);
//  this.hideAsset = ((!this.record.asset) || 'description' in this.record.asset)?false:true;
//  if (this.hideAsset) {
//  this.hideLocation = ((!this.record.locations) || 'description' in this.record.locations)?false:true;
//  } else {
//  this.hideLocation = true;
//  }
  },

  updateState: function(obj) {

    var statusEntry;
    if (this.formStatusSet) {
      statusEntry = this.getStatusEntry(obj.status_maxvalue);
    }else if (obj) {
      statusEntry = {'maxvalue':obj.status_maxvalue,
          'description':obj.status_description};
    }

    if (statusEntry !== null) {
      this.state = statusEntry;
      this.setBoolStatus(statusEntry);
    }

    if(this.record.status_maxvalue=='REVISED'){
      this.$.checkbox.readonly=true;
    }
  },

  _boolStateChange: function(newV, oldV) {

    if (oldV === undefined){
      return;
    }

    //make sure tooltip is closed 
    if($M.currentTooltip){
      $M.tooltip.hide();
    }

    if (this.skipBoolObserver){
      return;
    }

    var statusEntry;
    if (newV && newV === true) {
      statusEntry = this.getStatusEntry('active');
    }else {
      statusEntry = this.getStatusEntry('inactive');
    }

    if (statusEntry) {
      this.state = statusEntry;
    }

    if (this.record.status !== this.state.maxvalue) {    		
      var detail = {};
      detail.object = {'status': this.state.value};
      detail.props = 'status';
      detail.href = this.record.href;
      detail.originalStatus = this.record.status_maxvalue;
      detail.name = this.record.name;

      $M.toggleWait(true);
      this.fire('changeStatus',detail);
    }

  },

  setBoolStatus: function (domainEntry) {
    this.skipBoolObserver = true;
    if (domainEntry.maxvalue === 'ACTIVE') {
      this.booleanState = true;
    }else {
      this.booleanState = false;
    }
    this.skipBoolObserver = false;
  },

  getStatusEntry: function (/*String*/ maxval) {

    if (!maxval || maxval.length === 0) {
      console.error('Undefined maxval.');
      return null;
    }

    if (!this.formStatusSet){
      //console.error('No inspection form status domain set.');
      return null;
    }

    maxval = maxval.toUpperCase();
    var filterRes = this.formStatusSet.filter(function(o){return o.maxvalue === maxval;} );

    if (!filterRes){
      console.error('Problems to filter.');
      return null;
    }

    if (filterRes.length < 1){
      console.error('No values filtered with ' + maxval);
      return null;
    }

    return filterRes? filterRes[0] : null;
  },

  _preventPopup: function (ev) {
    ev.stopPropagation();
  },

  /**
   * Checks if mouse hovers for more than 350 milisecs
   */
  _showInfoHover: function(e) {

    var icon = e.currentTarget;
    var card = this;
    if(icon.hoverTimer){
      window.clearTimeout(icon.hoverTimer);
    }

    $j(icon).one('mouseleave', function(){
      if(icon.hoverTimer){
        window.clearTimeout(icon.hoverTimer);
      }
      $M.tooltip.hide();
    });

    icon.hoverTimer = setTimeout(function(){
      card._showInfo(e);
    }, 350);

  },

  mountRevision: function(record) {
    this._revision = $M.localize('uitext', 'mxapiinspection', 'revision_0',[record.revision]);
  },


  fillTooltip: function (record){
  },

  fillFormOptions: function (record){		
  },

  _showInfo: function(e) {		
    e.stopPropagation();
    var tooltip = this.$.info;
    $M.showTooltip(tooltip, e.target);
  },

  /**
   * Icon click handler
   * Show children detail in tooltip
   */
  _showChildren: function(e) {
    e.stopPropagation();
    this._loadChildrenDetails(e.target);
  },

  showRevisionAction: function(record) {
    if (record.status_maxvalue!=='ACTIVE' || (record.status_maxvalue==='ACTIVE' && record.hasrevision===true)){
      this.hideRevision=true;
    } else if((record.status_maxvalue=='ACTIVE' && record.originalformnum!=null)){
      this.hideRevision=false;
    } else {
      this.hideRevision=false;
    }
  },

  showDeleteAction: function(record) {
    if(record.originalformnum!=null && (record.status_maxvalue==='ACTIVE' || record.status_maxvalue==='INACTIVE')){
      this.hideDelete=true;
    } else if(record.hasrevision===false || record.status_maxvalue==='PNDREV'){
      this.hideDelete=false;
    } else {
      this.hideDelete=true;
    }
  },

  showEditAction: function(record) {
    if(record.status_maxvalue==='DRAFT' || record.status_maxvalue==='PNDREV'){
      this.hideEdit=false;
    } else {
      this.hideEdit=true;
    }
  },

  _showFormOptions: function (e) {

    e.stopPropagation();
    console.info('show options');

    this.showRevisionAction(this.record);
    this.showDeleteAction(this.record);
    this.showEditAction(this.record);

    var tooltipContent = this.$.formOptions;

//  $M.showTooltip(tooltipContent, e.target);
    this.$.tooltipInspection.show(tooltipContent, {'element':e.target,'stopMouseDown':true});
  },

  /**
   * Request inspection children details
   */
  _loadChildrenDetails: function(tooltipTarget) {

    //Activate spinner
    $M.toggleWait(true);

    //Set filter to match orgid, siteid and inspResultParent.referenceobjectid === parent
    var maximoQbe = new MaximoQbe();
    maximoQbe.setEquals('orgid', this.record.orgid)
    .setEquals('siteid', this.record.siteid)
    .setEquals('parent', this.record.referenceobjectid);

    var collection = this.$.inspchildrencollection;
    collection.queryWhere = maximoQbe.toJSON();

    //Fetch records
    var request = collection.refreshRecords();

    request.then(function(reqresp) {
      // Switch off spinner
      $M.toggleWait(false);
      var response = reqresp.response.member;

      var list = this._buildChildrenInfoList(response);
      this.set('childrenInfoList', list);

      var tooltip = this.$.children;
      $M.showTooltip(tooltip, tooltipTarget);

    }.bind(this), function(reason) {
      // Notify error
      console.error(reason);
      $M.notify(reason.message, $M.alerts.error);
      // Switch off spinner
      $M.toggleWait(false);
    });

  },

  /**
   * Prepare object list to show children details
   */
  _buildChildrenInfoList: function(response) {

    var list = [];

    for (var member of response) {
      list.push({
        referencenum: this.getElementIdentification(member),
        referencedesc: this.getElementDescription(member)
      });
    }

    return list;
  },

  _preview: function (e) {
    console.log('Open preview of ' + this.record.name);
    $M.notify('Under construction.', $M.alerts.warn);
    $M.tooltip.hide();
  },

  _revise: function (e) {
    console.log('Open edition of ' + this.record.name);
    this.fire('createRevision',this.record);
    $M.tooltip.hide();
  },

  _edit: function (e) {
    console.log('Open edition of ' + this.record.name);
    this.fire('editForm',this.record);
    $M.tooltip.hide();
  },

  highlight: function() {

    var card = this.$.inspectorListTile;

    card.classList.add('highlight');
    this.scrollIntoView(false);
    setTimeout(function() {
      card.classList.remove('highlight');
    }, 4000);


  },

  _duplicate: function (e) {
    this.fire('duplicate', this.record);
    $M.tooltip.hide();
  },

  _delete: function (e) {		
    var message = $M.localize('uitext','mxapiinspection','inspectionform_deleted');

    var self = this;
    var performDeleteOperation = function(overlayNode) {
      console.log('Perform delete');
      self.fire('delete', self.record);
    };

    $M.createOverlay(this.$.tileContainer, message, performDeleteOperation, this._revertDeleteOperation, this.undoIntervalSecs, null, null);

    //this.fire('delete',this.record);
    $M.tooltip.hide();
  },

  _performDeleteOperation: function(e){
    this.fire('delete',this.record);
  },

  _revertDeleteOperation: function () {
  },

  _config: function (e) {
    $M.tooltip.hide();
  },

  /**
   * Computed value for hideWO icon flag
   */
  _computedHideWOIcon: function(record) {
    var objtype = record.referenceobject;
    if (objtype === 'WORKORDER' || objtype === 'WOACTIVITY' || objtype === 'MULTIASSETLOCCI') {
      return false;
    }
    return true;
  },

  /**
   * Computed last completed status
   */
  _computedCompletedDate: function(record) {
    if ('inspresultstatus' in record && record.inspresultstatus.length) {
      var status = record.inspresultstatus[record.inspresultstatus.length-1];
      return status.changedate_localized;
    }
    return null;
  },

  /**
   * Computed stacked style
   */
  _computedIsStacked: function(record) {
    var value = false;
    if (record.referenceobject === types_.PARENTWO) {
      value = true;
    }
    this.enableSelection = !value;
    return value;
  },

  /**
   * Hidden function for tooltip attributes
   */
  _hideField: function(field) {
    if (!field) {
      return true;
    }
    return false;
  },

  /**
   * filter observer
   */
  _filterChanged: function(newV) {
    var tab;
    if (newV && newV.length) {
      tab = newV.toLowerCase();
      switch (tab) {
        case 'pending':
          this.$.maximo_inspection_tile_cretedOn.hidden=false;
          this.$.maximo_inspection_tile_dueDate.hidden=false;
          this.$.maximo_inspection_tile_completedDate.hidden=true;
          this.$.maximo_inspection_tile_status.hidden=true;
          break;
        case 'inprogress':
          this.$.maximo_inspection_tile_cretedOn.hidden=true;
          this.$.maximo_inspection_tile_dueDate.hidden=false;
          this.$.maximo_inspection_tile_completedDate.hidden=true;
          this.$.maximo_inspection_tile_status.hidden=true;
          break;
        case 'completed':
          this.$.maximo_inspection_tile_cretedOn.hidden=true;
          this.$.maximo_inspection_tile_dueDate.hidden=true;
          this.$.maximo_inspection_tile_completedDate.hidden=false;
          this.$.maximo_inspection_tile_status.hidden=true;
          break;
        default:
          this.$.maximo_inspection_tile_cretedOn.hidden=true;
        this.$.maximo_inspection_tile_dueDate.hidden=false;
        this.$.maximo_inspection_tile_completedDate.hidden=true;
        this.$.maximo_inspection_tile_status.hidden=false;
        break;
      }
    }
  },

  /**
   * long-touch listener
   */
  longTouchHandler: function(ev) {
	  this.set('tapStopper', true);
  },

  selectorShown: function(e) {
    e.stopPropagation();
    var ellipsis = this.$.inspectorTileListName.ellipsisSize;
    this.$.inspectorTileListName.ellipsisSize = ellipsis - 8;
  },

  selectorHidden: function(e) {
    e.stopPropagation();
    this.$.inspectorTileListName.ellipsisSize = this.ellipsis;
    this.$.inspectorTileListName.evaluateLabel();
  },

  /**
   * DEPRECATED
   */
  _formSelect : function(e){
    var record = this.record;
    record.page = 'start';
    this.fire('processFormSelection',record);
  },

  /**
   * DEPRECATED
   */
  _domainStatusChange: function(){
    //console.info('domain status is ');
    //console.log(this.formStatusSet);
  },
  
  /**
   * Function call when mouse hover out
   */
  _leavehover: function(e){      
      var icon = e.currentTarget;
      if(icon && $M.tooltip){
      $M.tooltip.hide();
   }
},

  
});
