/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2021
 */
Polymer({

  is: 'maximo-inspection-condition-card',

  behaviors: [BaseComponent, QuestionGroups, LookupReturnBehavior],

  listeners: {
    'maximo-lookup-selected'    : '_returnFromLookup',
    'maximo-typeahead-selected' : '_typeAheadSelectHandler'
  },

  /**
   * Fired when bin icon is clicked.
   *
   * @event delete-condition
   */

  properties : {

    /**
     * form object
     */
    formObject: {
      type: Object,
      value: null
    },

    /**
     * Main object to populate card
     */
    condition: {
      type: Object,
      value: null,
      observer: '_conditionChangeObserver'
    },

    /**
     * fieldOptions
     */
    options: {
      type: Array,
      value: []
    },

    /**
     * List of options formatted for mx-select
     */
    optionsListFormatted: {
      type: Array,
      computed: '_computeAvailableOptions(options)'
    },

    /**
     * available fields as target
     * srcfield already excluded from list
     */
    targets: {
      type: Array,
      value: [],
      observer: '_targetsChangeObserver'
    },

    /**
     * Text of the selected option
     */
    selectedOption: {
      type: String,
      value: null
    },

    /**
     * Object selected from typeahead or drawer
     */
    selectedTarget: {
      type: Object,
      value: {},
      observer: '_selectedTargetObserver'
    },

    /**
     * target description with sequence
     */
    displayTarget: {
      type: String,
      value: null
    },

    /**
     * Stores initial state for visible 
     * attribute for later comparisons
     */
    initialVisibleState: {
      type: Boolean,
      computed: '_storeOriginalVisibleState(condition)'
    },

    /**
     * Stores initial state for required 
     * attribute for later comparisons
     */
    initialRequiredState: {
      type: Boolean,
      computed: '_storeOriginalRequiredState(condition)'
    },

    /**
     * Card index
     */
    index: {
      type: Number,
      value: 0
    },

    /**
     * Tile title
     */
    title: {
      type: String,
      computed: '_computeConditionTitle(index, condition)'
    },

    /**
     * Sets the card as Read-only (for Saved Conditions)
     */
    isReadOnlyCard: {
      type: Boolean,
      computed: '_isReadOnlyCard(condition)'
    },

    /**
     * Sets the card as Read-only (for Saved Conditions)
     */
    isAvailableToSaveCard: {
      type: Boolean,
      value: false
    },	

    /**
     * Fixed condition piece
     * Set formnum and revision
     */
    staticClause: {
      type: String,
      computed: '_computeStaticClause(formObject)'
    },

	  /**
     * Variable condition piece
     * Set list of fieldnums
     */
    variableClause: {
      type: String,
      value: ''
    },

    /**
     * lookup filter
     */
    lookupWhereClause: {
      type: String
    },

    /**
     * List of fieldnums to be listed 
     * in lookup and typeahead
     */
    availableTargetsLookup: {
      type: Array,
      value: []
    },

    /**
     * Conditions from same src field
     */
    sameSrcConditionList: {
      type: Array,
      value: []
    },

    /**
     * Flag to help identify defaults 
     * at different status 
     * whether condition can be saved
     */
    hasDifferentDefaults: {
      type: Boolean,
      value: false
    },

    /**
     * required condition property
     */
    requiredCondition: {
      type: Boolean,
      value: false
    },

    /**
     * visible condition property
     */
    visibleCondition: {
      type: Boolean,
      value: false
    },
    
    /**
     * Array with Operators
     */
    operators: {
      type: Array,
      value: ["=", "!=", ">", ">=", "<", "<="]
    },
    
    /**
     * List of options formatted for mx-select
     */
    operatorsListFormatted: {
      type: Array,
      computed: '_computeAvailableOperators(operators)'
    },

    /**
     * List of options formatted for mx-select
     */
    secondOperatorsListFormatted: {
      type: Array,
      computed: '_computeAvailableOperators(operators)'
    },
    
    /**
     * Selected operator
     */
    operator1: {
      type: String,
      value: null
    },
  
    /**
     * Selected operator
     */
    operator2: {
      type: String,
      value: null
    },
    
    /**
     * Card for Single Option Field type
     */
    isNumericEntryCard: {
      type: Boolean,
      value: false,
      computed: '_isNumericEntryCard(condition)'
    },
    
    /**
     * Card for Numeric Response Field type
     */
    isSingleOptionCard: {
      type: Boolean,
      computed: '_isSingleOptionCard(condition)' 
    },
    
    /**
     * Value for Numeric input
     */
    number1: {
      	type: Number
    },

    /**
    * Value for Numeric input
    */
    number2: {
    	type: Number
    },
       
        
    /**
     * MaxValue for Numeric input
     */
    maxvalue: {
      	type: Number,
      	value: '99999999999'
      },        
        
        
    /**
    * MinValue for Numeric input
    */
    minvalue: {
    	type: Number,
    	value: '-99999999999'
    },  

        
    /**
     * Enable/Disable button to add a second numeric input in the range
     */
    isAvailableToAddSecondNumber: {
      type: Boolean,
      value: false
    },	
    
    /**
     * Show/Hide second Operator for Numeric inputs
     */
    showSecondNumberInput: {
      type: Boolean,
      value: false
    },	

    /**
     * Boolean to configure if the Inspector Feedback message is required
     */
    isShowMessageRequired: {
		  type: Boolean,
		  value: false
	  },
	
    /**
     * Inspector Feedback Message Field
     */
	  inspectorfeedback: {
      type: String,
      value: null
    },

    /**
    * Property used to check if the card is editable
    */
    editableCondition: {
      type: Boolean
    },

    /**
     * Boolean to require action property
     */
	  isRequireAction: {
      type: Boolean,
      value: false
	  },

    /**
     * Boolean to represent record's reject value
     */
    reject: {
      type: Boolean,
      value: false
    },

    /**
		 * Prop used to check if its a numeric validation
		 */
			showNumericValidation: {
        type: Boolean,
        value: false,
        computed: '_isNumericValidationCard(condition)'
      },

    /**
		 * Prop used to check if its a numeric condition
		 */
			showNumericCondition: {
        type: Boolean,
        value: false,
        computed: '_isNumericConditionCard(condition)'
      },

  },

  observers: [
	  '_computeAvailableToSaveCard(isReadOnlyCard, selectedOption, selectedTarget, hasDifferentDefaults, isNumericEntryCard, operator1, isShowMessageRequired, inspectorfeedback, isRequireAction, isNumericValidation)',	  
	  '_computeVariableClause(targets, selectedOption)',
    '_computeLookupClause(staticClause, variableClause)',
    '_handleReadOnlyCard(isReadOnlyCard)'
  ],

  created : function(){
  },
  ready : function(){
  },
  attached: function(){
  },

  /********** OBSERVERS begin **********/

  /**
   * Complex observer
   * Build lookup clause filter
   */
  _computeLookupClause: function(staticClause, variableClause) {
    let clause = null;
    let lookupFilter = null;

    if (staticClause) {
      clause = encodeURI(staticClause); 
    }

    if (variableClause) {
      if (clause) {
        clause = clause + encodeURI(' and ' + variableClause); 
      } else {
        clause = encodeURI(variableClause);
      }
    }

    lookupFilter = "oslc.where=" + clause;

    this.lookupWhereClause = lookupFilter;
  },

  /**
   *  Complex observer
   *  Enable Save button when Source and Target fields are selected - for Single Option responses
   *  or when Numeric Range exists and there's a Show Message value AND/OR a Target is selected,
   *  and the Visible/Required values are different from the original values from the Target Selected
   */
  _computeAvailableToSaveCard: function(isReadOnlyCard, selectedOption, selectedTarget, hasDifferentDefaults, isNumericEntryCard, operator1, isShowMessageRequired, inspectorfeedback, isRequireAction){


    let isAbleToSaveCondition = !Boolean(isReadOnlyCard);

    if (!Boolean(isNumericEntryCard)){
       isAbleToSaveCondition = Boolean(selectedOption) && isAbleToSaveCondition;
       
       isAbleToSaveCondition = (Boolean(selectedTarget) && Object.keys(selectedTarget).length !== 0 && selectedTarget.constructor === Object) && isAbleToSaveCondition;
       
       isAbleToSaveCondition = Boolean(hasDifferentDefaults) && isAbleToSaveCondition;
       
     } else {

      if (!Boolean(this.reject)){

        isAbleToSaveCondition = Boolean(this.operator1) && isAbleToSaveCondition;
    	
        let isSelectedTarget = (Boolean(selectedTarget) && Object.keys(selectedTarget).length !== 0 && selectedTarget.constructor === Object);
          
        isAbleToSaveCondition = isAbleToSaveCondition && ( (Boolean(isSelectedTarget) && Boolean(hasDifferentDefaults))
                                    || Boolean(isShowMessageRequired)
                                    || Boolean(isRequireAction));
  
        let validShowMessage = Boolean(isShowMessageRequired) ?  Boolean(this.inspectorfeedback) : true;
        
        isAbleToSaveCondition = validShowMessage && isAbleToSaveCondition;


      } else {

        isAbleToSaveCondition = Boolean(this.operator1) && isAbleToSaveCondition;

        isAbleToSaveCondition = isAbleToSaveCondition &&  Boolean(this.inspectorfeedback);

      }
        
    }
    
    this.isAvailableToSaveCard = isAbleToSaveCondition;
    
  },

  /**
   * Observer condition change
   */
  _conditionChangeObserver: function(newCondition, oldCondition) {
    if (newCondition) {
      
      this.requiredCondition = newCondition.required ? newCondition.required : false;
      this.visibleCondition = newCondition.visible ? newCondition.visible : false;
      
      this.operator1 = newCondition.operator1;
      this.number1 = newCondition.number1;
      this.operator2 = newCondition.operator2;
      this.number2 = newCondition.number2;
      
      this.showmessage = newCondition.showmessage;
      this.inspectorfeedback = newCondition.inspectorfeedback;
      
      this.requireaction = newCondition.requireaction;
      this.reject = newCondition.reject;
 
      this.selectedOption = newCondition.srctxtresponse ? newCondition.srctxtresponse : '';
      
      this.fieldtype = newCondition.fieldtype;

      this.reject = newCondition.reject;
      
      if (this.isReadOnlyCard) {
    	  if (Boolean(this.operator2)){
    	     this.showSecondNumberInput = true;
    	  }
      };
    }
  },
  
  /**
   * Observer targets change
   */
  _targetsChangeObserver: function(newTargetList, oldTargetList) {
    if (this.condition && this.isSavedCondition(this.condition)){
      let targetField = newTargetList.filter(target => {
        return target.inspfieldnum === this.condition.tgtfield;
      });
      if (targetField && targetField.length) {
        this.displayTarget = targetField[0].description;
      }
    }
  },

  /**
   * Observer target selected
   */
  _selectedTargetObserver: function(newObj, oldObj) {

    if (this.isReadOnlyCard) {
      return;
    }

    if (newObj && newObj.inspfieldnum) {
      this.condition.tgtfield = newObj.inspfieldnum;
      this.condition.tgtquestion = newObj.inspquestionnum;
      this.displayTarget = this.buildDisplayedTarget(newObj);
    } else {
      delete this.condition.tgtfield;
      delete this.condition.tgtquestion;
      this.displayTarget = null;
    }
    this._setDefaultsVisibleRequired(newObj);
  },

  /********** OBSERVERS end **********/
  /********** COMPUTED begin **********/

  /**
   * Compute variable clause for lookup
   */
  _computeVariableClause: function(targets, selectedOption) {
    let availableFields = [];
    let targetUsedwithSameOption = [];
    let fieldnumsClause;

    if (targets) {
      
      availableFields = targets.map( target => {
        return target.inspfieldnum;
      });

      if (selectedOption && this.sameSrcConditionList) {

        // Fetch target fields used in another condition with the same option
        for (let condition of this.sameSrcConditionList){
          if (selectedOption === condition.srctxtresponse) {
            targetUsedwithSameOption.push(condition.tgtfield);
          }
        }

        // Remove any target/field that has been selected with the same answer
        availableFields = availableFields.filter( targetFieldNum => {
          return !targetUsedwithSameOption.includes(targetFieldNum);
        });

      }
      
      // For Read-only cards, populate displayTarget field
      if (this.isReadOnlyCard) {
    	if (this.condition && this.isSavedCondition(this.condition)){
    	      let targetField = targets.filter(target => {
    	        return target.inspfieldnum === this.condition.tgtfield;
    	      });
    	      if (targetField && targetField.length) {
    	        this.displayTarget = targetField[0].description;
     	      }
    	    }   	  
      }

    }

    // When no available targets none should return from lookup, hence the  `inspfieldnum="0"`
    for(let i=0;i<availableFields.length;i++)
    {
    	if(!availableFields[i].includes('\"'))
    	{
    		availableFields[i] = '\"' + availableFields[i] + '\"';
    	}
    }
    
    fieldnumsClause = availableFields.length ? `inspfieldnum in [${availableFields.join()}]` : `inspfieldnum="0"`;

    this.variableClause = fieldnumsClause;
  },

  /**
   * Compute condition card title
   * @param {Number} index 
   */
  _computeConditionTitle: function(index, condition) {
    const visualIndex = ++index;
    if (condition.reject){
      return $M.localize('uitext','mxapiinspection','validation_index', [visualIndex]);
    } else if (!condition.reject){
      return $M.localize('uitext','mxapiinspection','condition_index', [visualIndex]);
    }
    
  },

  /**
   * Compute static part of condition
   */
  _computeStaticClause: function(formObject) {
    let staticClause = '';

    if (formObject && formObject.hasOwnProperty('inspformnum') 
        && formObject.hasOwnProperty('revision')) {

      staticClause = `inspformnum="${formObject.inspformnum}" and revision="${formObject.revision}"`;
    }
    return staticClause;
  },

  /**
   * Copy and parse field options for display
   * @param {Object} condition 
   */
  _computeAvailableOptions: function(options) {
    let availableOptions = [];
    if (options && options.length) {
      availableOptions = options.map(opt => {
        if (opt.description){
          //Single Options 
          return opt.description;
        } else {
          //Meter Options
          return opt;
        }
      });

      var isAlpha = false;
      for (i=0; i<availableOptions.length; i++){
        if (isNaN(availableOptions[i])){
          isAlpha = true;
        }
      }

      if (isAlpha){
        availableOptions.sort();
      } else {
        //Sort numeric fields
        availableOptions.sort(function(a, b){return a-b});
      }     
    }
    const entry = ':' + $M.localize('uitext','mxapiinspection', 'select_option');
    availableOptions.unshift(entry);

    return availableOptions;
  },

  /**
   * Get copy initial value of visible attribute
   * @param {Object} condition 
   */
  _storeOriginalVisibleState: function(condition) {
    return condition ? condition.visible : false;
  },

  /**
   * Get copy initial value of required attribute
   * @param {Object} condition 
   */
  _storeOriginalRequiredState: function(condition)  {
    return condition ? condition.required : false;
  },

  /**
   * Check if the Condition already exists
   * @param {Object} condition 
   */
  _isReadOnlyCard: function(condition) {
    return (this.isSavedCondition(condition) ? true : false);
  },

  /********** COMPUTED end **********/
  /********** HANDLERS begin **********/

  /**
   * Type Ahead list selection handler
   */
  _typeAheadSelectHandler: function(ev) {
    let payload = ev.detail;
    if (payload && payload.records && 
        typeof payload.records[Symbol.iterator] === 'function' && 
        payload.records.length){
      this.selectedTarget = payload.records[0];
    }
  },

  /**
   * Lookup list selection handler
   * Function to run when there's an item selected from the lookup/type ahead
   */
  _returnFromLookup: function(ev) {
    let payload = ev.detail;
    if (payload && payload.records && 
        typeof payload.records[Symbol.iterator] === 'function' && 
        payload.records.length){
      this.selectedTarget = payload.records[0];
    }
  },

  /**
   * Store visible state to condition object
   * @param {*} event 
   */
  _visibleStateChangeHandler: function(event) {
    let state = event.detail;
    // mx-select does not reflect value
    this.visibleCondition = state;

    if (state === false) {
      this.requiredCondition = false;
    }
    this._checkDefaultsInitialState();
  },

  /**
   * Store required state to condition object
   * @param {*} event 
   */
  _requiredStateChangeHandler: function(event) {
    let state = event.detail;
    // mx-select does not reflect value
    this.requiredCondition = state;

    if (state === true) {
      this.visibleCondition = true;
    }
    this._checkDefaultsInitialState();
  },

  /**
   * Handles trash icon
   * @param {Event} ev 
   */
  _trashIconClick: function(ev) {
    // validate if condition was not saved, show warning message
    if (this.condition && (this.condition.inspcascadeoptionid === undefined)){
      $M.showDialog(this, this.condition, 'Warning', 'maximo-conditions-warning-dialog', false);
    } else {
      this.deleteMe();
    }

  },

  /**
   * Handles Save 
   * Validation: For Numeric Response Fields, the Condition Card can have the "Show Message" AND/OR a Target Field
   * @param {Event} ev 
   */
  _saveClick: function(ev) {

	if (this.selectedTarget.inspfieldnum) {
       let generatedDisplayedTarget = this.buildDisplayedTarget(this.selectedTarget);
       // Check target field description
	    if (this.displayTarget !== generatedDisplayedTarget ) {
	      let msg = $M.localize('uitext','mxapiinspection','target_not_valid', [this.displayTarget]);
	      $M.alert(msg, $M.alerts.error);
	      return;
	    }
	    this.condition.tgtfield = this.selectedTarget.inspfieldnum;
	    this.condition.tgtquestion = this.selectedTarget.inspquestionnum;
	}    
    
    this.condition.srctxtresponse = this.isNumericEntryCard ? '' : this.selectedOption;
    
    this.condition.required = this.requiredCondition;
    this.condition.visible = this.visibleCondition;
    
    this.condition.operator1 = this.operator1;
    this.condition.operator2 = this.operator2;
    
    this.condition.number1 = this.number1;
    
    this.condition.number2 = (!Boolean(this.operator2)) ? null : this.number2;
    
    
    this.condition.showmessage = this.showmessage;
    this.condition.inspectorfeedback = this.inspectorfeedback;
   
    this.condition.requireaction = this.requireaction;

    if (this.isNumericValidation){
      this.condition.reject = true;
    };

    
    this.fire('saveConditionsForm', this);
  },

  /********** HANDLERS end **********/

  /**
   * Put sequence plus option description together
   */
  buildDisplayedTarget: function(targetObj) {
    try {
      var targetValue = this.targets.filter(target => {
    	        return target.inspfieldnum === targetObj.inspfieldnum;
      });      	 
      return targetValue[0].description; 
    } catch(error) {
      return null;
    }
  },

  /**
   * Verify if selected target properties matches:
   * - required
   * - visible
   * properties
   */
  _checkDefaultsInitialState: function() {
    let isDifferent = false;

    // Defaults stored in target selected object
    if (this.selectedTarget && Object.keys(this.selectedTarget).length !== 0 && this.selectedTarget.constructor === Object) {
      isDifferent = (this.requiredCondition !== this.selectedTarget.required) || (this.visibleCondition !== this.selectedTarget.visible);
    }

    // Set hasDifferentDefaults
    this.hasDifferentDefaults = isDifferent;
  },

  /**
   * Sort targets numerically
   */
  _sortTargets: function(a, b) {
    if (a.trim() === "" || a === undefined){
      a = "0";
    }

    if (b.trim() === "" || b === undefined){
      b = "0";
    }
    return parseFloat(a.substring(a.indexOf(":") + 1)) - parseFloat(b.substring(b.indexOf(":") + 1));
  },

  /**
   * Check if it is a persisted condition
   */
  isSavedCondition: function(condition) {
    return Boolean(condition && condition.inspcascadeoptionid);
  },

  /**
   * Emits delete event
   */
  deleteMe: function() {
    this.fire('delete-condition', this);
  },

  /**
   *  Setting the default values for Visible and Required toggles
   */
  _setDefaultsVisibleRequired: function(target){
    if (target) {
      this.requiredCondition = Boolean(target.required);
      this.visibleCondition = Boolean(target.visible);
      this.hasDifferentDefaults = false;
    }
  },
  
  /**
   * Check if the Condition already exists
   * @param {Object} condition 
   */
  _isNumericEntryCard: function(condition) {
    if ( (condition.fieldtype == "SE")  || 
       ( (condition.fieldtype == 'MM') && ( condition.metertype === "GAUGE" || condition.metertype === "CONTINUOUS")) 
    ){
      return true;
    }else{
      return false;
    }
  },
  
  /**
   * Check if the Condition already exists
   * @param {Object} condition 
   */
  _isSingleOptionCard: function(condition) {
	  if ((condition.fieldtype == "SO") || (condition.fieldtype == "MO") || ((condition.fieldtype == "MM") && (condition.metertype === "CHARACTERISTIC")) ||
        (condition.fieldtype == "SOD") ||  (condition.fieldtype == "MOD") 
    ){      return true;
    }else{
      return false;
    }
  },

  /**
   * Function to handle fields when the Operator is Selected 
   */
    _changedSelectedOperator: function(){
	  
       if ((this.operator1 === "=") || (this.operator1 === "!=") ){
		   this.isAvailableToAddSecondNumber = false;
       }else{
    	   this.isAvailableToAddSecondNumber = true;
       }	
	   
	   let availableSecondOperators = [];
       if (this.operators && this.operators.length) {
    	  availableSecondOperators = this.operators.filter(opt => {
              return ((opt !== "=") && (opt !== "!=") &&
            		  (opt !== this.operator1)
            		  //&& (this._validateOperators(opt))
                     );
          });    
       };
	      
	   for (i=0; i<availableSecondOperators.length; i++){
		   if ((availableSecondOperators[i] === ">") || (availableSecondOperators[i] === ">=")){
	    		if ((this.operator1 == ">") || (this.operator1 == ">=")){
	    			availableSecondOperators.splice(i,1);
	    		}
	    	} else if ((availableSecondOperators[i] === "<") || (availableSecondOperators[i] === "<=")){
	    		if ((this.operator1 == "<") || (this.operator1 == "<=")){
	    			availableSecondOperators.splice(i,1);
	    		}
	    	} 
	   };
	   this.secondOperatorsListFormatted = availableSecondOperators;
	      

    },	

    /**
     * Function to handle fields when the "add second operator" button is tapped
     */
    _showSecondNumberInput: function(){
	   	 this.number2 = 0;
    	 this.showSecondNumberInput = true;
    },
    
    /**
     * Copy and parse field options for display
     * @param {Object} condition 
     */

    _computeAvailableOperators: function(operators) {
      let availableOperators = [];
      if (operators && operators.length) {
    	  availableOperators = operators.map(opt => {
            return opt;
          });    
      }
      const entry = ':' + $M.localize('uitext','mxapiinspection', 'operator');
      availableOperators.unshift(entry);

      return availableOperators;
    },
    
    /**
     * Function to handle fields when the "remove second operator" button is tapped
     */
    _removeNumericInputIconClick: function(event){
    	 this.number2 = null;
    	 this.operator2 = null;
    	 this.showSecondNumberInput = false;
    	
    },
    
    /**
     * Store show a message state to condition object
     * @param {*} event 
     */
    _showMessageStateChangeHandler: function(event) {
      let state = event.detail;
      // mx-select does not reflect value
      this.showmessage = state;
      if (Boolean(this.showmessage)){
    	  this.isShowMessageRequired = true;
      } else {
    	  this.isShowMessageRequired = false;
      }
      
    },
    
    /**
     * Store require action state to condition object
     * @param {*} event 
     */
    _requireActionStateChangeHandler: function(event) {
      let state = event.detail;
      // mx-select does not reflect value
      this.requireaction = state;
      if (Boolean(this.requireaction)){
        this.isRequireAction = true;
      } else {
        this.isRequireAction = false;
      }
    },
    
    /**
     * Function to handle fields when the Card is Read-Only
     */
    _handleReadOnlyCard: function(isReadOnlyCard){

        if (Boolean(isReadOnlyCard) && !Boolean(this.operator2)){
         	     this.number2 = null;
         		 this.operator2 = null;
         		 this.showSecondNumberInput = false;
        }
    },  

  /**
   * Check if the Condition already exists
   * @param {Object} condition 
   */
  _isNumericValidationCard: function(condition) {
    let isNumeric = this._isNumericEntryCard(condition);

    return (isNumeric && condition.reject)? true : false;
  },

  /**
   * Check if the Condition already exists
   * @param {Object} condition 
   */
  _isNumericConditionCard: function(condition) {
    let isNumeric = this._isNumericEntryCard(condition);
    return (isNumeric && !condition.reject)? true : false;
  }

});
