/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2019
 */
Polymer({
	is: 'maximo-storeroom-issueitem',
  	behaviors: [BaseComponent,Storeroom],
  	listeners: {
  		'maximo-lookup-selected' : '_returnFromLookupIssue',
        'maximo-typeahead-selected' : '_returnFromLookupIssue',
        'asset-removed' : '_enableNext',
        'maximo-qrcode-result' : '_returnQRcode',
    	'maximo-qrcode-dialog-opened':'_adjustCamera'
    },
    
    properties : {
    	storeRowRecord : {
    		type : Object,
    		notify : true,
    		observer : '_setRowData'
    	},    	
    	invrecord : {
    		type: Object,
  			notify: true,
  			observer: '_setDefaultData'
        },        
        record : {
        	type: Object,
        	notify: true,
        },        
        rotatingItem : {
        	type: Boolean,
    		value: false
        },
        tositeid: {
        	type: String
        },
        conditioncode: {
			type: String
        },
        issuewonum: {
        	type: String
        },       
		lookupClause : {
			type : [],
			computed: '_getLookupClause(tositeid,conditioncode)',
			value: function(){
				return [];
			}
		},
		taskidClause: {
			type : [],
			computed: '_getTaskidClause(issuewonum)',
			value: function(){
				return [];
			}
		},
		locationClause: {
			type : [],
			computed: '_getLocationClause(tositeid)',
			value: function(){
				return [];
			}
		},
		issueqty : {
			type : Number,
			notify : true,
			value : 1.00,
			observer : '_setRotAssetBal'
		},
		schema : {
			type : Object
		},
		msgIntervalSecs : {
    		type: Number,
    		value: 3,
    		readOnly: false
    	},
    	rotatingassets : {
  			type: Array,
  			value: []
  		},
    },
    
    _closeFlyout : function() {
    	this.owner.close();
    },
    
    ready: function() {
		this._refreshIssueTypeDomain();
	},
    
    /**
     * Set data of current row
     */	
	_setRowData: function(){
  		if(this.storeRowRecord){
  			var rec = this.storeRowRecord;
  			this.storeRowRecord['itemnum'] = rec.itemnum?rec.itemnum:rec.invitemnum;
  			this.storeRowRecord['itemdescription'] = (rec.item && rec.item.description)?rec.item.description:rec.itemdescription;
  			this.storeRowRecord['binnum'] = rec.binnum?rec.binnum:rec.bincnt;
  			this.storeRowRecord['lotnum'] = rec.lotnum?rec.lotnum:rec.lotcnt;
  			this.$.itemnum.label = this.storeRowRecord.itemnum;
  			this.$.storeRowDesc.label = this.storeRowRecord.itemdescription;
  			this.$.storeBin.label = this.storeRowRecord.binnum;
  			this.$.storeLot.label = this.storeRowRecord.lotnum;
  			if(rec.itemcondition){
  			  this.$.storeCondition.label=rec.itemcondition[0].conditioncode;
  			}
  			$j(this.$.gldebitacct).find('input').prop('readonly',true);
  			this.async(function(){
  				if(this.flyoutowner){
  					this._setRowStyles(this.flyoutowner.selectedRow, $j(this).find('.flyoutcolumn'));
  				}  				
  			}, 100);
  		}  		
	},
  	_setDefaultData: function(){
  		var currentdate, currenttime;
		if(this.invrecord){
			this.record = this.invrecord.matusetrans[0];
			this.tositeid = this.record.tositeid;
			if(this.storeRowRecord && this.storeRowRecord.conditioncode){
				this.conditioncode = this.storeRowRecord.conditioncode;
			}				
			this.tositeClause = 'setupdata={%22matusetrans%22:{%22orgid%22:%22'+this.record.orgid+'%22}}';
			var dateformat = $M.getMaxauth().whoami.dateformat.toUpperCase();
			var timeformat = $M.getMaxauth().whoami.timeformat;
			var actualDate = this.record.actualdate_localized;
			if($M.isMobile()){
		    	 var indexOfSpace = actualDate.indexOf(' ');
		    	 currentdate = actualDate.substring(0, indexOfSpace);
		    	 currentdate = moment(currentdate, dateformat).format('YYYY-MM-DD');
		    	 currenttime = actualDate.substring(indexOfSpace, actualDate.length); 
		    	 currenttime = moment(currenttime, timeformat).format('HH:mm');
				  
		    } else {
	           	 if(moment(actualDate)._isValid){
	           		currentdate = moment(actualDate, dateformat).format(dateformat);
	           		currenttime = moment(actualDate).format(timeformat).toUpperCase();
	           	 }else{
	           		currentdate = moment(actualDate, dateformat).format(dateformat);
	           		currenttime = moment(actualDate, "D/M/YY h:mm a").format(timeformat).toUpperCase();
	           	 }	           	
			}
			this.$.actualdate.value = currentdate;
			this.$.actualtime.value = currenttime;
			if(this.storeRowRecord.item.rotating) {
				this.rotatingItem = true;
			}
		}		
	},	
	/**
     * Set values on select from lookup
     */
	_returnFromLookupIssue: function(e){
		var identifier = e.detail.identifier? e.detail.identifier : e.target.id;
		if(identifier ==='inventory_tosite'){
			this.tositeid = this.record.tositeid;              
		} else if(identifier ==='issuetoworkorder'||identifier ==='issuetolocation'||identifier ==='issuetoasset'){
			if(identifier === 'issuetoworkorder'){
				this.issuewonum = this.record.wonum;
			}
			this._getDefaultsLookupSelected(this.record,this.storeRowRecord, identifier);
		}
		this._enableNext();
	},
	 /**
	 * Get lookup where clause for Workorder/Aset Lookup
	 */
	_getLookupClause : function(tositeid, conditioncode){
		var tositeClause = '';
		if(tositeid && conditioncode){
			tositeClause = 'setupdata={%22matusetrans%22:{%22tositeid%22:%22'+tositeid
					+'%22,%22conditioncode%22:%22'+conditioncode+'%22}}';
		} else if(tositeid && !conditioncode){
				tositeClause = 'setupdata={%22matusetrans%22:{%22tositeid%22:%22'+tositeid+'%22}}';
		} else if(!tositeid && conditioncode){
				tositeClause = 'setupdata={%22matusetrans%22:{%22conditioncode%22:%22'+conditioncode+'%22}}';
		}
		return tositeClause;
    },
    /**
	 * Get wonum clause for workorder Lookup
	 */
	_getTaskidClause : function(issuewonum){
		var taskidClause = '';
		if(issuewonum !== undefined && issuewonum.length > 0){
			taskidClause = 'setupdata={%22matusetrans%22:{%22wonum%22:%22'+issuewonum+'%22}}';
		} 
		return taskidClause;
    },
    _getLocationClause: function(tositeid){
		var locationClause = 'domaininternalwhere=type=OPERATING,VENDOR,REPAIR';
		if(tositeid !== undefined && tositeid.length > 0){
			locationClause += '&oslc.where=siteid=%22'+tositeid+'%22';
		} 
		return locationClause;
    },
    _getDefaultsLookupSelected: function(record,invrecord, identifier) {
        var self=this;
        var resourceUri, actionData, responseProperties;
        actionData = {};
		responseProperties = 'matusetrans.location,matusetrans.assetnum,matusetrans.gldebitacct';
		actionData.issuetype = self._getIssueTypeValue('issuetypeissue');
		actionData.tositeid = record.tositeid;
		actionData.siteid = record.siteid;
		actionData.storeloc = record.storeloc;
		actionData.orgid = record.orgid;
		actionData.itemnum = record.itemnum;
		actionData.itemsetid = record.itemsetid;
		
		if(identifier === 'issuetoworkorder'){
			actionData.wonum = record.wonum;
		}
		if(identifier === 'issuetolocation'){
			actionData.location = record.location;
		}		
		if(identifier === 'issuetoasset'){
			actionData.assetnum = record.assetnum;
		}		
		
		actionData._action = 'Add';
       	 
		if(record.wonum || record.location || record.assetnum){
            this.$.issueitemresource.resourceUri = invrecord.matusetrans_collectionref;  
            $M.toggleWait(true);
            
            this.$.issueitemresource.updateRecord([actionData],responseProperties, true, false, {'setvaluemode':true}).then(
            	function(result) {
            		$M.toggleWait(false);
		        	if(result) {
		        		self.$.issuetolocation.value = result.matusetrans.location;
		        		self.$.issuetoasset.value = result.matusetrans.assetnum;
		        		if(!(self.$.issuetoworkorder.value && self.$.gldebitacct.value) || (identifier === 'issuetoworkorder')){
		        			self.$.gldebitacct.value = result.matusetrans.gldebitacct;
		        		}
		                self._enableNext();                     		 
		         	}                
               }, function(error) { 
	        	 	$M.toggleWait(false);
	        	 	$M.showResponseError(error);                                 
               });
         }       	
	 },
	 _setRotAssetBal : function() {
    	 var rotatingasset = this.$$('#rotating_asset');
    	 if(rotatingasset){
    		 rotatingasset.quantity = Math.round(this.issueqty);
        	 rotatingasset._setQuantity();
    	 }    	 
    	 this._enableNext();
     },
     
     _setIssueQty : function() {
    	 this.issueqty = this.$.quantity.$.valueElement.value? parseInt(this.$.quantity.$.valueElement.value, 10) : 0;
     },
     
     _setIssueQtyVal : function() {
    	 this.issueqty = this.$.quantity.value? parseInt(this.$.quantity.value, 10) : 0;
     },
     /**
      * Enable/disable save button
      */
 	_enableNext : function(){
 		var issueIcon = this.$.issue_item_button;
 		var issueQty = this.issueqty;	
 		issueQty = parseFloat(parseFloat(issueQty).toFixed(2));
 		var issueObj;

 		var gldebitacct = this.$.gldebitacct.value;
 		$j(issueIcon).attr('disabled', true);
 		
 		if(this.storeRowRecord && issueQty > 0 && gldebitacct && gldebitacct!=='Unspecified' && gldebitacct.trim()!=='' && !(gldebitacct.includes('?'))) {
 			var rotassetpage = this.$$('#rotating_asset');
 			if(this.storeRowRecord.item.rotating && rotassetpage){
 				if(issueQty === rotassetpage.rotatingassets.length){
 					$j(issueIcon).attr('disabled', false);
 				}
 			} else {
 				$j(issueIcon).attr({'disabled': false});
 			}
 		}
 	},
 	
 	_issueItem : function(){
 		var collectionToUpdate = $M.getGlobalResource('storeroomdetailcollection');
 		var issueqty = parseFloat(this.$.quantity.$.valueElement.value ? this.$.quantity.$.valueElement.value : this.$.quantity.value);
		var message = $M.localize('uitext','mxapiinventory','itemissued',[issueqty, this.invrecord.itemnum, this.invrecord.issueunit]);
		var tableRow;
        if(this.flyoutowner){
        	tableRow = this.flyoutowner.selectedRow;
        	if(this.storeRowRecord.item.rotating) {
            	this._issueRotatingItem(this, collectionToUpdate, tableRow, message);
            } else {
            	this._issueNonRotatingItem(this, collectionToUpdate, tableRow, message);
            }
        }
 	},
 	
 	_getActualDateTime: function(self){
 		var actualDate;
 		if($M.isMobile() && self.$.actualdate.value){
 			actualDate = moment(self.$.actualdate.value,'YYYY-MM-DD').format($M.getMaxauth().whoami.dateformat.toUpperCase()) + ' ' + self.$.actualtime.value;
        }
        else if(self.$.actualdate.value && self.$.actualtime.value && self.$.actualtime.value.split(' ')[0] !== '0:00'){
        	actualDate = self.$.actualdate.value + ' ' + self.$.actualtime.value;
        }
        else if(self.$.actualdate.value){
        	actualDate = self.$.actualdate.value;
        }
 		return actualDate;
 	},
 	
 	_issueRotatingItem : function(self, collectionToUpdate, tableRow, message){
 		var issueRotInv = [];
        var issueInv ={};
        var selfobj = self;
        var requestHeaders = {};
        issueInv.matusetrans = [];
        issueInv.matusetrans[0] = {};
        $M.toggleWait(true);
        $j(this.$.quantity).find('input').blur();
        
        var rotassetpage = this.$$('#rotating_asset');
		if(rotassetpage && rotassetpage.rotatingassets.length > 0) {
			rotassetpage.rotatingassets.forEach(function(rotasset, index){
				issueInv ={};
				issueInv.matusetrans = [];
				issueInv.matusetrans[0] = {};
				issueInv.href = self.invrecord.href;
				issueInv.matusetrans[0].issuetype = self.invrecord.matusetrans[0].issuetype;
				if(self.$.inventory_tosite.value){
					issueInv.matusetrans[0].tositeid = self.$.inventory_tosite.value.trim();
				}
				if(self.$.issuetoworkorder.value){
					issueInv.matusetrans[0].wonum = self.$.issuetoworkorder.value.trim();
				}				
				if(self.$.issuetotaskid.value){ 
					issueInv.matusetrans[0].taskid = self.$.issuetotaskid.value;
				}
				if(self.$.issuetoasset.value){
					issueInv.matusetrans[0].assetnum = self.$.issuetoasset.value.trim();
				}
				issueInv.matusetrans[0].actualdate = self._getActualDateTime(self);
				if(self.$.issuetoperson.value){
					issueInv.matusetrans[0].issueto = self.$.issuetoperson.value.trim();
				}
				if(self.$.enteredby.value){
					issueInv.matusetrans[0].enterby = self.$.enteredby.value.trim();
				}
				if(self.$.memo.value){
					issueInv.matusetrans[0].memo = self.$.memo.value.trim();
				}
				if(self.$.storeCondition.label)	{
					issueInv.matusetrans[0].conditioncode=self.$.storeCondition.label;
				}
				if(self.storeRowRecord.binnum) {
					issueInv.matusetrans[0].binnum = self.storeRowRecord.binnum;
				}
				issueInv.matusetrans[0].quantity = 1;
				issueInv.matusetrans[0].orgid = self.invrecord.matusetrans[0].orgid;
				issueInv.matusetrans[0].location = self.$.issuetolocation.value;
				issueInv.matusetrans[0].rotassetnum = rotasset.assetnum;
				issueInv.matusetrans[0].storeloc = self.invrecord.matusetrans[0].storeloc;
				issueRotInv.push(issueInv);
			});
		}
		
		requestHeaders.allornothing = true;
		requestHeaders.method = 'BULK';
		var responseProperties = collectionToUpdate.attributeNames;
		this.$.issueitemresource.resourceUri = self.invrecord.href.slice(0,self.invrecord.href.indexOf('mxapiinventory')+14);
		this.$.issueitemresource.updateRecord(issueRotInv, responseProperties, null, true, requestHeaders).then(
			function(result) {
				selfobj._issueCallback(collectionToUpdate, tableRow, message, selfobj);
			 }, function(error) {
		         $M.showResponseError(error);
		         $M.toggleWait(false);
		         selfobj.owner.close();
		     	});
 	},
 	
 	_issueNonRotatingItem : function(self, collectionToUpdate, tableRow, message){
 		var issueInv ={};
		issueInv.matusetrans = [];
        issueInv.matusetrans[0] = {};
        var selfobj = self;
        $M.toggleWait(true);
        $j(this.$.quantity).find('input').blur();
        
        issueInv.href = self.invrecord.href;
		issueInv.matusetrans[0].issuetype = self.invrecord.matusetrans[0].issuetype;
		if(self.$.inventory_tosite.value){
			issueInv.matusetrans[0].tositeid = self.$.inventory_tosite.value.trim();
		}
		if(self.$.issuetoworkorder.value){
			issueInv.matusetrans[0].wonum = self.$.issuetoworkorder.value.trim();
		}		
		if(self.$.issuetotaskid.value) { 
			issueInv.matusetrans[0].taskid = self.$.issuetotaskid.value;
		}
		issueInv.matusetrans[0].assetnum = self.$.issuetoasset.value ? self.$.issuetoasset.value.trim() : '';		
		issueInv.matusetrans[0].actualdate = self._getActualDateTime(self);
		if(self.$.issuetoperson.value){
			issueInv.matusetrans[0].issueto = self.$.issuetoperson.value.trim();
		}
		if(self.$.enteredby.value){
			issueInv.matusetrans[0].enterby = self.$.enteredby.value.trim();
		}
		if(self.$.memo.value){
			issueInv.matusetrans[0].memo = self.$.memo.value.trim();
		}		
		if(self.$.storeCondition.label)	{
			issueInv.matusetrans[0].conditioncode=self.$.storeCondition.label;
		}
		if(self.storeRowRecord.binnum) {
			issueInv.matusetrans[0].binnum = self.storeRowRecord.binnum;
		}	
		issueInv.matusetrans[0].quantity = self.$.quantity.value;
		issueInv.matusetrans[0].orgid = self.invrecord.matusetrans[0].orgid;
		issueInv.matusetrans[0].location = self.$.issuetolocation.value;
		issueInv.matusetrans[0].storeloc = self.invrecord.matusetrans[0].storeloc;
        
        self.$.issueitemresource.resourceUri = self.invrecord.href;
        self.$.issueitemresource.updateRecord(issueInv, 'itemnum', true).then(
				function(result) {
					selfobj._issueCallback(collectionToUpdate, tableRow, message, selfobj);				
				}, function(error) {
					$M.toggleWait(false);
					selfobj.owner.close();
					$M.showResponseError(error);					
		});
 	},
 	
 	_issueCallback : function(collectionToUpdate, tableRow, message, selfobj){
		var completeOverlay = function(){
			collectionToUpdate.refreshRecords();
		}
		$M.toggleWait(false);
        selfobj.owner.close();
		var overLay =  $M.createOverlay(tableRow, message, completeOverlay,null,selfobj.msgIntervalSecs, selfobj.parentElement.parentElement, null, true);
		$j(tableRow).find('maximo-button[id^=overlayButton]').remove()
		selfobj._preventScrollToTop();
	},
	
	/**
	 *Prevent scroll to top after collection refresh
	 */
	_preventScrollToTop: function(){
		var ironList = this.flyoutowner.$.storeroomTable.$.rowRepeat;
		ironList.scrollOffset = ironList._scrollTop;
	},
	 _returnQRcode: function(event) {
		    var url=this.storeRowRecord.matusetrans_collectionref;
	 		this._getToFromBinLotbarCodeVal(url,this,event);
	 	},
	 	_getToFromBinLotbarCodeVal:function(url,self,event){
	 		
	 		var bincollection = Polymer.Base.create('maximo-collection', {
	 			'id':  'binCollection',
	 			'pageSize' : '1', 
	 		}); 
	 		var issuecomponent ='';
	 		var issueLookup = $M._lookups['rotatingasset'];
	 	    self.issueURI =  url+'/zombie/getlist~'+issueLookup.getList +'?oslc.select='+issueLookup.attributeNames+'&setupdata={%22matusetrans%22:{%22tositeid%22:%22'+self.tositeid+'%22}}';
	 	     
	     	 var maximoQbe = new MaximoQbe();
	     	 maximoQbe.setEquals('assetnum',event.detail.value);
	 		 qbeObject = maximoQbe.toJSON();
	 		 bincollection.queryWhere = qbeObject;     	
	     	 bincollection.collectionUri=self.issueURI;    	
	     	 bincollection.refreshRecords().then(function(record){					
	 		 event.detail.records=[bincollection.collectionData[0]];
	 			 if(event.detail.identifier==='toRotate' && event.detail.records[0]!=undefined){
	 				 
	  				var rotatingassetArray= $j(self).find('#rotating_asset')[0].rotatingassets;
	                var exist=false;
	                rotatingassetArray.forEach(function(item,index){
	                  if(item.assetnum==event.detail.records[0].assetnum){
                   	    exist=true;
                        return exist;
                       }
	                 });
	                 
	                 if(!exist){
	               	  $j(self).find('#rotating_asset')[0].rotatingassets.push(event.detail.records[0]);   
	                  }              
	                 $j(self).find('maximo-datalist')[0].items= $j(self).find('#rotating_asset')[0].rotatingassets;
	                 $j(self).find('maximo-datalist')[0].refresh();
	                 $j(self).find('#rotating_asset')[0].selqty = rotatingassetArray.length;
	                 $j(self).find('#rotating_asset')[0]._setQuantity();
	                 }
	 		  else{ 	
	  			$M.notify($M.localize('uitext', 'mxapisr', 'NoResultsFound'), $M.alerts.warn);
	 		 }
	 			self._enableNext();
	 	});
	 	},
	 	_adjustCamera:function(){
	          $j('maximo-workscape').find('#flyout').css('z-index', '20000');
	    }

	 	
});

