/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2019
 */
Polymer({
	is: 'maximo-supervisor-approve',
  	behaviors: [BaseComponent,Workorder],
	listeners: {
  		'changeStatusApprove': '_changeStatusApprove',
  		'approveSelected':'_approveSelected',
  		'workorder-status-changed':'_notifyParent'
	},
	properties : {
		/* query for the collection */
		selectedQueryName: {
			type: String
		},
		/* identifies current panel */
		panelName: {
			type: String
		},
		/* Panel record count */
		recordCount: {
			type: Number,
			notify: true
		},
		/**
	     * Sorting categories map
	     */
    	sortSet: {
    		type: Object,
    		value:  []     
    	} ,
    	
    	/**
    	 * wo Status  domain 
    	 */
    	statusDomain: {
    		type: Object 
    	},

    	/**
    	 * wo Class  domain
    	 */
	    classDomain: {
    		type: Object 
    	},
	},
  	
	/**
  	 * Triggered when collection is refreshed.
  	 */
  	_handleRecordDataRefreshed: function(){
  		this.recordCount = this.$.approvewocollection.totalCount;
  		this.$.approveWorkPanel.toggleLoading(false);
  	},
  	
	/**
	 * Triggered by listener to change status of record to APPROVE.
	 */
	_changeStatusApprove : function(e){
		
		var record = e.detail;
		var collection = this.$.approvewocollection;
		var resource = this.$.approvewoResource;
		var panel = this.$.approveWorkPanel;
		var ironList = this.$.approveWorkPanelIronList;
		var status = this._getExternalValueForWOstatus('APPR');
		this._changeStatusOnPanelCards(record,status,panel,resource,collection,ironList);
	},	

	/**
	 * Triggered by listener to change status of record to APPROVE when multiple records are selected.
	 */
	_approveSelected: function(e){
		var self = this;
		var collection = self.$.approvewocollection;
		var resource = self.$.approvewoResource;
		var panel = self.$.approveWorkPanel;
		var ironList = self.$.approveWorkPanelIronList;
		var status = this._getExternalValueForWOstatus('APPR'); //Get the locale value for the status
		var selectedRecords = panel.getSelectedRecords();
		var count = selectedRecords.length;
		panel.toggleLoading(true);

		var message = $M.localize('uitext','mxapiwodetail','wosApproved',[count]);
		var performChangeStatus = function(overlayNode) {
			self._changeStatusOnPanelCards(selectedRecords,status,panel,resource,collection,ironList);
			panel.immediateDeactivateMultiSelect();
		};
		
		var undo = function() {
			panel.toggleLoading(false);
		};
			
		$M.notify(message, $M.alerts.info, performChangeStatus,undo);
	},
	
	/**
	 * add listener  on search bar
	 */
	attached: function() {
		var that = this;

		this.sortSet = [
			{ key:'wonum', displayName: $M.localize('uitext', 'mxapiwodetail', 'Workorder')},
			{ key:'worktype', displayName: $M.localize('uitext', 'mxapiwodetail', 'Type')},
			{ key:'wopriority', displayName: $M.localize('uitext', 'mxapiwodetail', 'Priority')}, 
			{ key:'location.description', displayName: $M.localize('uitext', 'mxapiwodetail', 'Location')},
			{ key:'assetnum', displayName: $M.localize('uitext', 'mxapiwodetail', 'Asset')},
			{ key:'schedstart', displayName: $M.localize('uitext', 'mxapiwodetail', 'Scheduledstart')}
			];
	},
	ready : function() {
		// add menu items for the selected items
		var menuItems = [];
		menuItems.push({label:$M.localize('uitext','mxapiwodetail','approveSelected'),event:'approveSelected',sep:true});
		if(this.$.approveWorkPanel && $j(this.$.approveWorkPanel).find('maximo-panelmenu').length > 0){
			$j(this.$.approveWorkPanel).find('maximo-panelmenu')[0].setMenuItems(menuItems);
		}
	},
	
	/** 
	 * Set the query filter.
     * STATUS='WAPPR' AND  SITEID=:SITEID AND  SUPERVISOR=:SUPERVISOR
     * This will be used together with saved query to improve the performance
	 */
	_updateCollectionFilter : function() {
		var maxauth = $M.getMaxauth();
		var status = [];
		status = this._getDomain(this.statusDomain, 'WAPPR') ;
	    maximoQbe = new MaximoQbe();
		maximoQbe.setIn('status',status).
				  setEquals('siteid','-dfltsite-').
		          setEquals('supervisor','-personid-');
		qbeObject = maximoQbe.toJSON();
		
		if(!this.$.approvewocollection.queryWhere && this.$.approvewocollection.queryWhere!=''){
			this.$.approvewocollection.queryWhere = qbeObject;	
		}
				
		this.$.approveSort.collection = this.$.approvewocollection;
	},
	

	/** 
	 * apply the query filter and refresh the collection.
	 */
	refreshCollection: function() {
		this._updateCollectionFilter();
		//Set collection page size for mobile device
		if($M.isMobile()){
			this.$.approvewocollection.pageSize='5';	
		}
		this.$.approvewocollection.refreshRecords();
	},
	
	/** 
	 * retrieve a specific domain
	 */
	_getDomain: function(domainList , mxvalue) {
		var result =[];
		if(domainList && domainList[0] && domainList[0].synonymdomain) {
			domainList[0].synonymdomain.forEach(function(domain){
				if(domain.maxvalue === mxvalue ){
					result.push(domain.value) ;
				}
			}, this);
		}
		
		return   result;
	},
	
	/** 
	 * fire an event to force the workscape update the owner panel
	 */
	_notifyParent: function(e) {
		var self = this;
		// To avoid multiple collection refreshes for same collection
		var ownerPanelList = [];
		var monitorPanelList = [];
		if($j.isArray(e.detail) && e.detail.length >= 1){
			e.detail.forEach(function(record, index){
				if((record.ownerperson && record.ownerperson.displayname) || record.ownergroup || record.assignedownergroup){
					monitorPanelList.push(record);
				} else {
					ownerPanelList.push(record);
				}
			});			
		} else {
			this._updatePanel(e.detail);
			return;
		}
		if(ownerPanelList.length > 0){
			this.fire('update-owner-collection');
		}
		if(monitorPanelList.length > 0){
			this.fire('update-monitor-collection');
		}		
	},
	
	/**
	 * To refresh owner or monitor panels
	 */
	_updatePanel : function(record){
		if((record.ownerperson && record.ownerperson.displayname) || record.ownergroup || record.assignedownergroup){
			this.fire('update-monitor-collection'); 
		} else {
			this.fire('update-owner-collection'); 
		}
	},
	
	/**
     * Enable wait layer, initiated on refresh of collection
     */
    _loading : function(){
       this.recordCount = -1;
       this.$.approveWorkPanel.toggleLoading(true);
    }
});
