/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2018,2019
 */

Polymer({
    is: 'maximo-login',
    resourceComments: ['\/\/START NON-TRANSLATABLE','\/\/END NON-TRANSLATABLE'],
    properties: {

    	username: {
    		type: String,
    		value: ''
    	},

    	password: {
    		type: String,
    		value: ''
    	},
    	signInError: {
    		type: Boolean,
    		value: false
    	},
    	error: {
    		type: Boolean,
    		value: false
    	},
    	phoneScreen: {
    		type: Boolean,
    		value: false,
    		observer: 'phoneScreenChanged'
    	},

    	loginMaterialPanelClass: {
    		type: String
    	},
		personData: {
			type: Object,
			notify: true					
		},
      	maxauth: {
      		type: Object
      	},
      	resources: {
      		type: Boolean,
      		value: false
      	},
      	showing: {
      		type: Boolean,
      		value: false
      	},
      	_inactivityTimeoutStart: { //in seconds
      		type: Number
      	},
      	_userActive: {
      		type: Boolean,
      		value: false
      	},
      	_userActivityThreshold: {
      		type: Number,
      		value: 120
      	},
      	_lastPing: {
      		type: Number,
      		value: 0
      	},
      	_showing: {
      		type: Boolean,
      		value: true
      	},
      	
    	_firstAttempt: {
      		type: Boolean,
      		value: true
      	},
      	
        apikey: {
	        type: String,
	        value: ''
	    },
      	
      	_titleLabel: {
      		type: String 
      	},
      	
      	_loginLabel: {
      		type: String 
      	},
      	
      	_userNameLabel: {
      		type: String 
      	},
      	
      	_signInLabel: {
      		type: String 
      	},
      	
      	_errorLabel: {
      		type: String 
      	},
      	
      	_passwordLabel: {
      		type: String 
      	},
      	
      	_guestEmail: {
      		type: String 
      	},
      	
      	_guestInfo: {
      		type: String 
      	},
      	
      	_guestError: {
      		type: String 
      	},
      	
    },
    behaviors: [BaseComponent, HTMLEncoder],
    
    configureLabels: function(){
    	this._titleLabel =this.localize('uitext', 'login','main_title');
    	this._loginLabel =this.localize('uitext', 'login','log_in');
    	this._userNameLabel =this.localize('uitext', 'login','user_name');
    	this._signInLabel =this.localize('uitext', 'login','sign_in');
    	this._errorLabel =this.localize('uitext', 'login','error');
    	this._passwordLabel =this.localize('uitext', 'login','password');
    	this._guestEmail =this.localize('uitext', 'login','guestEmail');
    	this._guestInfo =this.localize('uitext', 'login','guestInfo');
    	this._guestError =this.localize('uitext', 'login','guestError');
    },
    
    ready: function(){

    },
	attached: function(){
		
		$M.login = this;
		this.loadResources();
		this.async(function(){
			$j(this).find('#copyyear').text((new Date()).getFullYear());
		},	 100);
		
		this.lobbyLoadResources().then(function(value) {
			var lobby = document.createElement('maximo-lobby-page');
			lobby.setAttribute('id', 'lobby');
			Polymer.dom(this.root).appendChild(lobby);
			Polymer.dom.flush();
			
		}.bind(this));
	},
    loadResources: function(lang, retry){
		if(this.resources){
			this.handleLogin();
			return;
		}
   		var findLang = lang;
   		if(!findLang) {
   			findLang = lang==='en'? '': $M.lang==='en'?'':($M.lang.toLowerCase()==='en-us'?'':'/'+$M.lang);
   		}
    	var maximoLoginProp = this.resolveUrl('..')+'maximo-login/translation'+findLang.toLowerCase()+'/resources.json';
    	var login = this;
    	$j.ajax({
			url: maximoLoginProp,
			headers: {
			    'Accept-Language': lang
			},
			lang: findLang,
			data: {
				format: 'text'
			},
			error: function(jqXHR, textStatus, errorThrown ) {
				if(retry){
					console.log('An error has occured while loading the login resources for '+this.lang+'. Reverting to English.');	
					login.loadResources('en');
				}
				else {
					login.loadResources('/'+this.lang.substring(1).split('-')[0], true);
				}
			},
			dataType: 'text',
			success: function(data, textStatus, jqXHR) {
				login.resourceComments.forEach(function(comment){
					data = data.replace(new RegExp(comment, 'g'),'');
				});
				data = JSON.parse(data);
	    		$M._storeResources('uitext','login', data);
				login.saasAndFederal();
				login.resources = true;
				login.handleLogin(true);
				login.configureLabels();
			},
			type: 'GET'
		});
    },
    
	  /**
     * 
     * @param {*} lang
     * @param {*} retry
     */
    lobbyLoadResources: function(lang, retry){
      return new Promise(function(resolve, reject) {
        if(this.resources){
          resolve();
        }
    
        var findLang = lang;
        if(!findLang) {
			findLang = lang === 'en' ? '' : $M.lang === 'en' ? '' : ($M.lang.toLowerCase() === 'en-us' ? '' : $M.lang);
		}

		var maximoLobbyProp = this.resolveUrl('..')+'maximo-lobby-page/translations/' + findLang.toLowerCase() + '/resources.json';
        var lobby = this;
        $j.ajax({
          url: maximoLobbyProp,
          headers: {
            'Accept-Language': lang
          },
          lang: findLang,
          data: {
            format: 'text'
          },
          dataType: 'text',
          error: function(jqXHR, textStatus, errorThrown ) {
            if(retry){
              console.log('An error has occured while loading the lobby resources for ' + this.lang + '. Reverting to English.');	
              lobby.loadResources('en');
            } else {
              lobby.loadResources(this.lang.substring(1).split('-')[0], true);
            }
            reject();
          },
          success: function(data, textStatus, jqXHR) {
            $M._storeResources('uitext', 'lobbypage', JSON.parse(data));
            lobby.resources = true;
            resolve();
          },
          type: 'GET'
        });
      }.bind(this));
    },
    
    saasAndFederal: function(){
    	var maximoLoginProp = this.resolveUrl('..')+'maximo-login/maximo-login-properties.json';
    	var login = this;
    	$j.ajax({
			url: maximoLoginProp,
			headers: {
			    'Accept-Language': login.lang
			},
			data: {
				format: 'text'
			},
			error: function(jqXHR, textStatus, errorThrown ) {
				login.setSaasAndFederal(false,false);
			},
			dataType: 'json',
			success: function(data, textStatus, jqXHR) {
				login.setSaasAndFederal(data.isSaas,data.isFederal);
			},
			type: 'GET'
		});

    },
    
    /**
     * Check if customer is SAAS and federal 
     */
    setSaasAndFederal: function(isSaas,isFederal) {
    	if(isSaas){
    		if(isFederal){
    			$j(this).find('#saasLabel').$.label.innerHTML = this.localize('uitext','login','federal'); 
    		} else {
    			$j(this).find('#saasLabel').$.label.innerHTML = this.localize('uitext','login','nonfederal'); 
    		}
    	} else {
    		$j(this).find('#saasNotice').css({'display':'none'});
    	}
    },
	handleChange: function(e){
		if(e.keyCode===13){
			this.handleLogin(false);
		}
	},
	handleLogin: function(silent){
		this.silent = silent===true;
		if(!silent){
			$M.toggleWait(true);
		}
		var params = false;
		var username = $j(this).find('#uname_input').val();
		var password = $j(this).find('#pwd_input').val();
		if(window.urlParams.username && window.urlParams.password) {
			params = true;
			username = window.urlParams.username;
			password = window.urlParams.password;
		}
		$M.addToMaxauth('username', username);
		$j(this).find('#message').attr('hidden','true');
		this.signInError = false;
		sessionStorage.setItem('lastLoginAttempt', (new Date()).getTime());
		this.$.authenticator.login(username, password);
  },

  handleGuestChange: function(e){
	  e.stopPropagation();
		if(e.keyCode===13){
			 e.preventDefault();
			 this.handleGuestLogin(false);
		}
	},
	
  /**
   * 
   */
  handleGuestLogin: function(silent){
     var apikey = this.apikey,
     guestid = $j(this).find('#guestEmail').val().toUpperCase();
    
   

    this.silent = (silent === true);
    this.signInError = false;

    if(!silent){
      $M.toggleWait(true);
    }

    $M.addToMaxauth('apikey', apikey);
    $M.addToMaxauth('guestid', guestid);
	$M.addToMaxauth('username', 'mguest');

    sessionStorage.setItem('lastLoginAttempt', (new Date()).getTime());
    
    	 this.$.authenticator.guestLogin(apikey, guestid);
  
   
  },

  handleLoginError: function(e){
		$j(this).find('#message').attr('hidden','true');
		if(!this.silent || $M.getMaxauth().blueid_url){
			if(!this._firstAttempt){
				this.signInError = true;
			}
		}
		this._firstAttempt = false;

		var errorResponse = e.detail.request.__data__.response,
			inProgressLabel = "botc#SubscriptionInprogress",
			inProgressCode = "BMXAA9705E";

		// The response of the login error comes with an object, but browsers parse this object in a different way:
		//   - Google Chrome reads the data as e.detail.request.__data__.response.Error.message
		//   - Firefox reads the data as e.detail.request.__data__.response["oslc:Error"]["oslc:message"]
		var chromeMessage = (typeof(errorResponse.Error) !== 'undefined' && errorResponse.Error.message && (errorResponse.Error.message == inProgressLabel || errorResponse.Error.message.startsWith(inProgressCode))),
			firefoxMessage = (typeof(errorResponse["oslc:Error"]) !== 'undefined' && errorResponse["oslc:Error"]["oslc:message"] && (errorResponse["oslc:Error"]["oslc:message"] == inProgressLabel || errorResponse["oslc:Error"]["oslc:message"].startsWith(inProgressCode))),
			tenantInProgress = (chromeMessage || firefoxMessage);

		if (tenantInProgress) {
			errorResponse.errorparams.splice(0, 1);
			this.handleSubscriptionInProgress({ e: { detail: { subscriptionData: errorResponse.errorparams }}});
		} else {
			if($M.getMaxauth().blueid_url && e.detail.request.__data__.status === 403){
				$j(this).find('.loginpanel').attr('hidden','true');
				localStorage.setItem('urlToGo', $M.getMaxauth().blueid_dashboard);
				if ($M.getMaxauth().blueid_url !== undefined){
					localStorage.setItem('inprocess', 'true');
				} else {
					localStorage.setItem('inprocess', 'false');
				}
	/*			if (e.detail.request.__data__.xhr.response !== undefined &&
						e.detail.request.__data__.xhr.response.Error !== undefined &&
						(e.detail.request.__data__.xhr.response.Error.message === 'botc#SubscriptionInprogress' || 
								e.detail.request.__data__.xhr.response.Error.message.startsWith('BMXAA9705E'))){
					localStorage.setItem('inprocess', 'true');
				}else{
					localStorage.setItem('inprocess', 'false');
				}*/
				setTimeout(function() {
					console.log('---------------------');
					console.log(e);
					document.location='logout.html';
				}, 5000);
			}
			this.silent = false;
			this.showMe();
			this.showMessage(localStorage.getItem('loginMessage'));
			localStorage.removeItem('loginMessage');
			$M.toggleWait(false);
			this.async(function(){
				$j('app-wait').attr('hidden',true);
			},100);
		}
	},
	handleLoginSuccess: function(e){
		var locationHash = sessionStorage.getItem('locationHash');
		if (locationHash) {
      		if (!window.location.hash) {
       	 		sessionStorage.removeItem('locationHash');
       		 	sessionStorage.setItem('hashOverride', 'true');
       	 		window.location.hash = locationHash;

       	 		var locationSearch = sessionStorage.getItem('locationSearch');
        		if (locationSearch) {
        	  		sessionStorage.removeItem('locationSearch');
        	  		window.location.search = locationSearch;
        	 		return;
      	  		}

      	 		window.location.reload();
       	    	return;
   	   		}
   		 }	
		if(window.performance.navigation.type !== 1){
			this.toggleLoading(true);
		}
		this.silent = false;
		
		var lobby = this.$$("#lobby");
		if(lobby){
			lobby.parentNode.removeChild(lobby);
		}

		var login = this;
		var maxauth = $M.getMaxauth();
		
		this._userActivityThreshold = ((maxauth.sessiontimeout - maxauth.inactivetimeout)/3);
		if(this._userActivityThreshold > 120){ //ping server at least every minute
			this._userActivityThreshold = 120;
		}
		
		this._lastPing = this._getCurrentTimeInSeconds();
		this._startInactivityTimer();
		
		this.$.userInfoResource.resourceUri = $M.configureURLContext(maxauth.baseUri + '/oslc/whoami?addapps=1&wcsyscfg=1');
		this.$.userInfoResource.loadRecord().then(function(result){
			$M.loadWhoAmI(result);
		}, 
		function(error, response) {
			console.warn('Could not fetch user information.');
			console.error(login.localize('uitext','login','unknown_error',[]));
		});
	},
	loadResourcesAndWorkscape: function(){
		var requestedWorkCenter = $M.$.contextAppRouter.data.workcenter;
		if(requestedWorkCenter){
			if($M.hasAccessToWorkscape(requestedWorkCenter.toLowerCase())){
				$M.toggleWait(true);
				$M.loadWorkscape(requestedWorkCenter);
				return;
			}
			else if(Object.keys($M.getAllowedWorkscapes()).length > 2) { //not test and not welcome
				this._offerBaseRedirect();
				return;
			}
		}
		var defwork = $M.getDefaultApplicationId(true);
		if(defwork){
			this.changeToApp(defwork);
		}
		else {
			if(requestedWorkCenter){
				this._offerBaseRedirect();
			}
			else {
				window.location =   $M.getMaxauth().baseUri.split('imeguest')[0];
			}
		}
	},
	_offerBaseRedirect: function(){
    	this.toggleLoading(false);
		var message = $M.localize('uitext','login','no_access_default');
		var dialog = $M.confirm(message, function(response, e){
			if(response === true){
				// IJ44469
				// this.changeToApp($M.getDefaultBaseApplicationId(true));
				window.location = window.location.protocol + '//' + window.location.host + window.location.pathname;
			}
			else {
				this._cancelLogin();
			}
		}, this, 1, [0,1]);
		this.async(function(){
	    	$M.toggleWait(false);
		}, 200);

		// IJ44469
		this.async(function(){
			dialog.querySelector('.dialogCloseButton').hidden = true;
		}, 100);
	},
	_noAllowedWorkScapes: function(defaultApp){
		var message = $M.localize('uitext','login','noworkscape');
		$M.confirm(message, function(response, e){
			if(response === true){
				window.location = $M.getMaxauth().baseUri;
			}
			else {
				this._cancelLogin();
			}
		}, this, 1, [0,1]);

	},
	_cancelLogin: function(){
		$M.toggleWait(true);
    	this._clearForm();
    	this.$.authenticator.logout();
	},
	/**
	 * Determines workcenter or application in maximo an changes to it.
	 */
	changeToApp: function(id){
		this.toggleLoading(true);
		if(id.toLowerCase()==='startcntr'){
			$M.openStartCenter();
			return;
		}
		var application = $M.getApplication(id);
		var type = application && application.apptype?application.apptype:'RUN';
		switch (type){
			case 'RUN':
/*				var location = $M.getMaxauth().baseUri.split('imeguest')[0]; 
				if(id && id.length>0){
					location += '/ui/?event=loadapp&value='+id;
				}*/
			 
                var classicLink = $M.getMaxauth().maximoclassiclink;
                if (classicLink && classicLink.length > 0) {
                	classicLink = classicLink + '/ui/?event=loadapp&value='+id;
    				window.location = classicLink;
                } else {
					var location = $M.getMaxauth().baseUri.split('imeguest')[0]; 
					if(id && id.length>0){
						location += '/ui/?event=loadapp&value='+id;
					}
					window.location = location;
                }
				break;
			default:
				if(application.custapptype==='REACT'){
					window.location = $M.getMaxauth().baseUri+$M.getMaxauth().apiContext+'/graphite/'+id.toLowerCase();
					break;
				}
				$M.currentWorkscape = 'login';
			 	$M.set('route.path', '/'+id.toLowerCase());
				break;
		}
	},
	showMessage: function(key){
		if(!key || key === undefined){
			return;
		}
		this.signInError = false;
		this.error = true;
		var param = this.localize('uitext', 'login', 'contact_admin');
		var string = this.localize('uitext', 'login', key, [param]);
		if(string.length>30 && window.innerHeight<=400){
			string = string.substring(0,85)+'...';
		}
		if(string !== undefined && string !== 'undefined'){
			this.$$('#message').label = string;
			this.$$('#message').removeAttribute('hidden');
		}
	},
	
	phoneScreenChanged: function()
	{
		if (this.phoneScreen === true)
		{
			this.loginMaterialPanelClass = 'loginMaterialPanelPhoneScreen';
		}
		else
		{
			this.loginMaterialPanelClass = 'loginMaterialPanel';
		}
	},
	toggleLoading: function(on){
		if($j(this).find('.loginpanel').css('display') !=='none'){
			$M.lockWait = on;
			$M.toggleWait(on);
		}
	},
	hideMe: function(){
		$j(this).find('.loginpanel').css({'opacity':'0'});
		$j(this).find('.loginpanel').css({'display':'none'});
		if($M.workScape){
			$M.workScape.hidden=false;
		}
	},
	showMe: function(){
		if($M.workScape){
			$M.workScape.hidden=true;
		}
    	
		$j(this).find('#pwd').attr('type', 'password');
		var login = this;
		this.async(function(){
			$j(login).find('.loginpanel').css({'display':'inherit'});
			$j(login).find('.loginpanel').css({'opacity':'1'});				
		});
	},
	
	/**
	 * 
	 * @param {*} details 
	 */
	handleSubscriptionInProgress: function(e) {
		var subscriptionData = e.detail.subscriptionData,
				totalTime = subscriptionData[0],
				elapsedTime = subscriptionData[1];

		$M.toggleWait(false);
		this.$$("#lobby").toggleVisibility(true);
		this.$$("#lobby").updateProgress(totalTime, elapsedTime);

		this.async(function(){
			this.handleLogin(true);
		}, 30000);
	},

	listeners: {
		'subscription-in-progress': 'handleSubscriptionInProgress',
		'login-success': 'handleLoginSuccess',
		'login-error': 'handleLoginError',
		'login-user-activity': '_userActivity',
		'login-error-message': '_updateGuestErrorMessage',
	},

    _clearForm: function(){
    	$j(this).find('input').val('');
    },
    _getCurrentTimeInSeconds: function(){
    	return Math.floor(new Date().getTime() / 1000);
    },
    _checkInactivityTimer: function(){
    	var maxauth = $M.getMaxauth();
    	if(!maxauth){
    		return;
    	}
    	var elapsed = (this._getCurrentTimeInSeconds() - this._inactivityTimeoutStart ) + 1;
    	
    	var remaining = maxauth.sessiontimeout - elapsed;

    	if(this._userActivityCheck()){
    		return;
    	}
    	
    	if($M.debug){
	    	if(!this.timer){
	    		this.timer = document.createElement('div');
	    		document.body.appendChild(this.timer);
	    		$j(this.timer).css({'position':'absolute','z-index':'10000','background-color':'#fff','top':'50px','color':'red','border':'1px solid #999'});
	    	}
	    	this.timer.innerHTML = HTMLEncoder.sanitize(remaining);
    	}
    	if(elapsed >= (maxauth.sessiontimeout - maxauth.inactivetimeout)){
    		if(remaining <= 0){
    			window.clearTimeout(this.inactivityInterval);
    			this.inactivityInterval = null;
    			if(this.timeoutWarnDialog){
    				this.timeoutWarnDialog.close();
    			}
    			$M.toggleWait(true);
    			this.async(function(){
    				$M.workScape.timeout();	
    			}, 100);
    			return;
    		}
    		else {
		    	if(this.timeoutWarnDialog){
		    		$j('.sessionRemaining').html(HTMLEncoder.sanitize(Math.abs(remaining)));
		    	}
		    	else {
		    		var count = remaining;
		    		if(!this.inactivityMessage){
			    		this.inactivityMessage = $M.localize('uitext','mxapibase','inactivity_warn', [count]);
			    		this.inactivityQuestion = $M.localize('uitext','mxapibase','inactivity_stay');
		    		}
		    		this.timeoutWarnDialog = $M.confirm('', function(){this.timeoutWarnDialog = null; this._startInactivityTimer(true);}, this, 0, [0], null, 'none');
		    		$j(this.timeoutWarnDialog).find('.dialogIcon').hide();
		    		$j(this.timeoutWarnDialog).find('.dialogCloseButton').hide();
		    		$j(this.timeoutWarnDialog).find('.header').hide();
		    		var login = this;
		    		//change to 1 second interval so we can refresh the countdown
	    			window.clearTimeout(this.inactivityInterval);
	    			this.inactivityInterval = null;
	    			this.async(function(){
	    				$j(this.timeoutWarnDialog).find('div.alertMessageCell').html(HTMLEncoder.sanitize(this.inactivityMessage + '<br>'+ this.inactivityQuestion));
	    			},200);
		    		this.inactivityInterval = window.setInterval(function(){
		        		login._checkInactivityTimer();
		        	}, 1000);
		    		$j(this.timeoutWarnDialog).on('mousedown mouseup tap', function(e){
		    			e.stopPropagation();
		    		});
		    	}
    		}
    	}
    },
    _resetInactivityStart: function(){
    	var maxauth = $M.getMaxauth();
    	if(!maxauth){
    		return;
    	}
    	var currentTime = this._getCurrentTimeInSeconds();
    	//adjust by 5 seconds to avoid request missing window at the end of countdown
    	this._inactivityTimeoutStart = currentTime - 5;
    	if(this.inactivityInterval){
    		window.clearTimeout(this.inactivityInterval);
    		this.inactivityInterval = null;
    	}
    	return currentTime;
    },
    _pingServer: function(maxauth){
    	if(this.inactivityInterval){
    		window.clearTimeout(this.inactivityInterval);
			this.inactivityInterval = null;	
    	}
		this._userActive = false;
		this.$.keepAlive.resourceUri = $M.configureURLContext( maxauth.baseUri + '/oslc/ping/');
    	var login = this;
    	//do this before ping to avoid a few second time mismatch problem. Better to timeout a few seconds early than leave user without a session.
		this._lastPing = this._getCurrentTimeInSeconds();
		this.$.keepAlive.loadRecord().then(function(result){
			login._startInactivityTimer(false);
		});
    },
    _startInactivityTimer: function(ping){
    	var maxauth = $M.getMaxauth();
    	if(!maxauth || !maxauth.sessiontimeout || maxauth.sessiontimeout === 0){
    		return; //do nothing
    	}
   	
    	if(ping){
    		this._pingServer(maxauth);
    		return;
    	}
    	
    	this.sessionTimeoutStart = this._resetInactivityStart();
    	
    	var login = this;
    	if(!this.inactivityInterval){
        	this.inactivityInterval = window.setInterval(function(){
        		login._checkInactivityTimer();
        	}, 1000);
    	}
    	if(!this.bodyInactivityEvents){
    		this.bodyInactivityEvents = true;
    		
    		$j('body').on('tpae-activity', function(e){
				login.fire('login-user-activity', {type:e.type});
    		});
    		
    		$j('body').on('tap', function(e){
				login.fire('login-user-activity', {type:e.type});
    		});

    		$j('body').on('keydown', function(e){
    			login.fire('login-user-activity', {type:e.type});
    		});
    		
    		$j('body').on('mouseup', function(e){
    			login.fire('login-user-activity', {type:e.type});
    		});
    		
    		$j('body').on('mousedown', function(e){
    			login.fire('login-user-activity', {type:e.type});
    		});
    		
    		$j('body').on('mousemove', function(e){
    			if(!login.timeoutWarnDialog){
    				login.fire('login-user-activity', {type:e.type});
    			}
    		});
    	}
    },
    _userActivity: function(e){
    	this._userActive = true;
    },
    _userActivityCheck: function(){
    	var maxauth = $M.getMaxauth();
    	if(!maxauth){
    		return;
    	}
		if(this.stopUserActivityEvents || this.timeoutWarnDialog){ //only one every 2 seconds 
			return;
		}
		this.stopUserActivityEvents = true;
		var login = this;
		this.activityTimer = setTimeout(function(){
			window.clearTimeout(login.activityTimer);
			login.stopUserActivityEvents = false;
		}, 2000);
		var elapsed = this._getCurrentTimeInSeconds() - this._lastPing;
		if((elapsed > this._userActivityThreshold) && this._userActive){
			this._startInactivityTimer(true);
			return true;
		}
		return false;
    },
    _handleUserDataRefreshed: function(e){
    	$M.userInfo = this.personData[0];
    	sessionStorage.setItem('userinfo', JSON.stringify($M.userInfo));
    	this.loadResourcesAndWorkscape();
    },
    _handleUserDataError: function(){
    	this._offerBaseRedirect();
    },
    /**
	 * Function will insert datepicker language data as per user locale.
	 * Function will return closest possible data based upon user locale provided, for example- will take care scenarios like es, es-US, es-Br also.
	 * It will also remove any invalid locale entry made while fetching the data.
	 * It will return default language as english, if no matching entry found even with or without country code.
	  */ 
    datepickerLanguageData: function(locale) {           
        var headData = document.getElementsByTagName('head')[0];
        var scriptData = document.createElement('script');
        scriptData.charset = "UTF-8";
        var langURL = 'script/libraries/jquery/jquery-ui/datepicker/datepicker-';
        if(locale === 'en-US' || locale === 'en') {
        	return $j.datepicker.regional[locale];
        }
        else {
        	scriptData.type = 'text/javascript';
            scriptData.src = langURL + locale + '.js';
        	headData.appendChild(scriptData);
        }
        var self = this;
        this.async(function(){
        	if($j.datepicker.regional[locale]) {
            	return $j.datepicker.regional[locale];
            }
            if(locale.indexOf("-") > 0) {
                headData.removeChild(scriptData);
                self.datepickerLanguageData(locale.split("-")[0]);
            }else {
            	headData.removeChild(scriptData);
            	return $j.datepicker.setDefaults($j.datepicker.regional[""]);
            }
		},200);
	},
	
	
	_updateGuestErrorMessage: function(event){
		if(event.detail.Error && event.detail.Error.message){
			if(event.detail.Error.reasonCode){
				this._guestError = event.detail.Error.message.replace(event.detail.Error.reasonCode+' - ','');
			}else{
				this._guestError = event.detail.Error.message;
			}
		}
	},
});
