/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */

Polymer({
	is: 'maximo-numeric-spinner',
	behaviors: [BaseComponent, DirtyBehavior],
	
	properties : {
		/**
		 * Set the label value
		 */
		label: {
			type : String,
			value : ''
		},
		
		/**
		 * Set the value displayed. It can be only numbers integer or decimal.
		 */
		value:{
			type: Number,
			notify: true,
			value:0,
			observer: '_validateRange'
		},
		
		/**
		 * Set the display value
		 */
		displayValue:{
			type: Number,
			notify: true
		},
		
		
		/**
		 * maximum value allowed
		 */
		max:{
			type: Number
		},
		
		/**
		 * minimum value allowed
		 */
		min:{
			type: Number
		},
		
		/**
		 * Step used to increase/decrease
		 */
		step:{
			type: Number,
			value : 1
		},
		
		/**
		 * flag to decimal number
		 */
		isDecimal: {
			type: Boolean,
			value: false,
			observer: '_setPrecision'
		},
		
		/**
		 * precision float
		 */
		precision:{
			type: Number,
			value : 2
		},
		
		/**
		 * indicate if is required
		 */
	    required: {
	         type: Boolean,
	         value: false,
	         notify: true
	    },
		
	    /**
		 * Time duration in Hours:Minutes
		 */
		duration:{
			type: Boolean,
			value: false,
			notify: true
		},
		
		/**
		 * Time interval used to increase/decrease in minutes
		 * Default 30 minute interval
		 */
		interval:{
			type: Number,
			value : 30
		},
		
		/**
		 * Hour - part of the duration
		 */
		hour:{
			type: Number,
			value : '00',
			notify: true
		},
		
		/**
		 * Minute - part of the duration
		 */
		minute:{
			type: Number,
			value : '00',
			notify: true
		},
		
		/**
		 * indicate if the component is configured
		 */
		_isReady:{
			type: Boolean,
			value: false,
		},
		
		/**
		 * decimal separator
		 */
		_decimalSeparator:{
			type : String,
			value : '.',
			observer: '_configurePattenr'
		},
	   /**
		 * Set how many characters the field allows
		 */
		maxlength: {
		  type: Number
		},
		
		/**
		 * true if negative values are allowed.
		 */
		allowNegative:{
		  type: Boolean,
		  value: false
		},
		
		/**
		 * Calculates negative time values.
		 */
		negativeTime: {
			type: Number,
			value:'00'
		},
		
		/**
		 * display placeholder
		 */
		showPlaceholder: {
			type: Boolean,
			value:false,
			observer: '_clearValue'
		},
		/**
		 * Get current cursor position in value field
		 */
		curCaretPos: {
			type: Number
		},
		/**
		 * Get current cursor position in hour field
		 */
		curHrCaretPos: {
			type: Number
		},
		
		_pattern: {
			type: String,
			value:'-?[0-9]*'
		}
		
		
	},
	
	
	/**
	 * display placeholder
	 */
	_clearValue: function() {
		if(this.showPlaceholder && this.value==0){
			this.value=undefined;
		}
	},
	
	/**
	 * set width
	 */
	created: function() {
	
		
	},
	
	/**
	 * set placeholder and validate Special Characters
	 */
	ready: function() {
		this.$.valueElement.addEventListener("keyup", clearSpecialCharacters );  
		
		if(this.showPlaceholder && this.value==0){
			this.value=undefined;
		}
		this._decimalSeparator = this._getDecimalSeparator();
	},
	
	/**
	 * set width
	 */
	attached: function() {
		this._updateDisplayValue();
		var self =this;
		/*
		 * Bound multiple events i.e onKeyPress, onPaste, onDrop to restrict special
		 * characters and alphabets to hour and minute input.
		 */
		$j(this.$.hourDurationElement).add(this.$.minuteDurationElement).on('keypress input', function(evt){
			self.curHrCaretPos = this.selectionStart;
			return self.allowNegative?isNumberKey(evt):onlyNumberValue(evt);			
		});
		this._configurePattenr();
	},
	
	
	/**
	 * decrease value
	 */
	_decrease: function(){
		if(this.duration){
			var value = (this.allowNegative) ? parseInt(this.negativeTime) - this.interval : parseInt(this.minute) - this.interval;
  			this._calculateTimeDuration(value);
  		} else {
  			
  			if(this.showPlaceholder && this.value == undefined ){
  				this.value = 0;
   				this._updateDisplayValue(true);
			}else{
				if(this.value==undefined){
	   				this.value = 0;
	   			 } 
				 if(this.isDecimal) {
  	  				 this.value =  parseFloat(parseFloat(parseFloat(this.value) - this.step).toFixed(this.precision));
  	  			 }else{
  	  				 this.value = this.value - this.step;
  				 }
			}
  		}
  	},
   	
  	/**
	 * increase value
	 */
  	_increase: function(){
  		if(this.duration){
  			var value = (this.allowNegative) ? parseInt(this.negativeTime) + this.interval : parseInt(this.minute) + this.interval;
  			this._calculateTimeDuration(value);
  		} else {
  			
  			if(this.showPlaceholder && this.value == undefined ){
  				this.value = 0;
   				this._updateDisplayValue(true);
			}else{
				if(this.value==undefined){
	   				this.value = 0;
	   				 
	   			 } 
   				 if(this.isDecimal) {
   					this.value =  parseFloat(parseFloat(parseFloat(this.value) + this.step).toFixed(this.precision));
   	  			 }else{
   	  			    this.value = this.value + this.step;
   	  			 }
			}
  			 
  		}
  	},
  	  	
  	/**
	 * check if the value is valid
	 */
  	_validateRange: function(){
  		 
  		if(this._isReady){
  			this._validateValue();
  		}else{
  			var self = this;
  			this.async(function(){
  	  			self._validateValue();
  	  		}, 200);
  		}
  		
  	},
  	
	/**
	 * check if the value is valid
	 */
  	_validateValue: function(){
		if(this.value !== undefined ){
  			if(this.duration){
  			  this._updateTime();
  			}else{
  				// using != for null check
  				if(this.min != undefined &&  this.value < this.min){
  				  this.value = this.min; 
  				  return;
  				}
  				// using != for null check
  				if(this.min != undefined &&  this.value > this.max){
  				  this.value = this.max; 
  				 return;
  				}
  				if(this.isDecimal && isNaN(this.value) && this.value !== undefined){
  					this.value =  parseFloat(parseFloat(this.value.toString().replace(/,/g,'.')).toFixed(this.precision));
  					this.clearDirtyStatus();
  					return;
  				}
  			this._updateDisplayValue();
  			}
  		}else{
  			this._updateDisplayValue();
		}
		
		this._isReady=true;
  	},
  	
  	
  	/**
	 * update display value for decimal and integer number
	 */
  	_updateDisplayValue: function( forceUpdate){
  			this.displayValue=undefined;
  	  		if(this.duration){
  				this._updateTime();
  			}else if(this.isDecimal) {
  				if(this.showPlaceholder && this.value == undefined ){
  					this.displayValue = '';
  					return;
  				}
  				if( this.value == undefined ){
  	  				this.value = 0;
  	  				return;
  				}
  				
  	  			var decimalString = this.value.toString().replace(/,/g,'.');
  	  			var precisionSrt=decimalString.split( '\.')[1];
  	  	    	decimalString = this.value.toString();
  				var completePrecisionSrt='';
  				if (precisionSrt){
  					for (var index=0 ; index<(this.precision-precisionSrt.length);index++){
  						completePrecisionSrt+='0';
  					}
  				}else{
  					completePrecisionSrt = this._decimalSeparator;
  					for (var index=0 ; index < this.precision; index++){
  						completePrecisionSrt+='0';
  					}
  				}
  				if(forceUpdate){
  					this.displayValue = '';
  				}
  				this.displayValue = (this._getDecimalSeparator() === ',') ? (decimalString + completePrecisionSrt).toString().replace(/\./g,',') : (decimalString + completePrecisionSrt);
  			}else{
  				if(this.showPlaceholder && this.value == undefined ){
  					return;
  				}
  				if(this.value==undefined){
  	   				this.value = 0;
  	   				return;
  	   			 } 
  				 
  				if(forceUpdate){
  					this.displayValue = '';
  				}
  				this.displayValue = (this._getDecimalSeparator() === ',') ? this.value.toString().replace(/\./g,',') : parseInt(this.value);
  			}
  	  		//Set cursor position
  	  		if(this.curCaretPos){
  	  			var elem = this.$.valueElement;
  	  			elem.setSelectionRange(this.curCaretPos, this.curCaretPos);
  	  		}	
			
  	},
  	
  	
  	/**
	 * clear the field if the user set any special Character
	 */
  	_clearSpecialCharacters: function(){
  		 if(this.$.valueElement.value.length > 0){ 
  		    let specialCharacters='!@#$%^&*()+=[]\\\';,/{}|\":<>?` àèìòùãõâêîôûÀÈÌÒÙÂÊÎÔÛÀÕÃˆ~˜`'; 
  		  
  		    let newValue=this.$.valueElement.value;
  		    let clearText = false;
  		    for (let index = 0 ; index < specialCharacters.length ; index++ ){
  		    	if(newValue.indexOf(specialCharacters[index])!=-1){
  		    		clearText = true;
  		    		break;
  		    	}
  		    }
  		    if(clearText){
  		    	this.$.valueElement.value = '';
  		    }
  		 }
  	},
  	
  	/**
	 * update value - manual input
	 */
  	_updateValue: function(){
  	  //	this._clearSpecialCharacters();
  	    var valElem = this.$.valueElement;
  	    this.curCaretPos = valElem.selectionStart;
  	    var updatedVal = valElem.value;
  	    
  	    if(updatedVal){
  	    	updatedVal = updatedVal.split(this._decimalSeparator)[0];
  			if(updatedVal.indexOf('0') === 0){
  				this.curCaretPos = this.curCaretPos -1;
  			}
  	    }
  		if(this.$.valueElement.value == '' || this.$.valueElement.value == '-' || this.$.valueElement.value == undefined){
  			if(this.showPlaceholder ){
  				this.value = undefined;
  				this._updateDisplayValue(true);
  				return;
			}else{
				this.value = 0;
			} 
  		}
		
  		if(this.isDecimal) {
  			if(this.$.valueElement.value != '' && this.$.valueElement.value != '-'){
  				this.value = parseFloat ( parseFloat (this.$.valueElement.value.toString().replace(/,/g,'.')).toFixed(this.precision));
  			}
  			this._updateDisplayValue(true);
  			
		}else if(this.duration){
			this._updateTime();
  		}else{
  			if(this.$.valueElement.value != '' && this.$.valueElement.value != '-'){
  				this.value =  parseInt (this.$.valueElement.value);
  			}else{
  			// this.$.valueElement.value='0';
  			}
  			this._updateDisplayValue(true);
		}
  		
  	},
  	
  	/**
	 * get decimal separator
	 */
  	 _getDecimalSeparator:function(){
  		 var result = '.';
  		 try{
  			 let decimalNumber = 1.1;
  			 let decimalString = decimalNumber.toLocaleString();
  			 if ($M.getUserInfo() && $M.getUserInfo().defaultLcale){
				var locale = $M.getUserInfo().defaultLcale;
				decimalString = (locale.includes("_") || locale.includes("-")) ? decimalNumber.toLocaleString(locale.slice(0, 2)) : decimalNumber.toLocaleString(locale);
  			 }
  			 if(decimalString.indexOf('.') != -1){
  				 result =  '.';
  			 }else{
  				 result =  ',';
  			 }
  		 }catch(err) {
  			 console.log('Could not get the decimal separator using the locale. ');
  		 }
  		 return result;
	},

  	
  	/**
	 * update the display with the right precision
	 */
  	_setPrecision: function(){
  		if(this.isDecimal && this.value==0){
  			var completePrecisionSrt='0'+ this._getDecimalSeparator();
			for (var index=0 ; index<this.precision;index++){
				completePrecisionSrt+='0';
			}
			this.$.valueElement.value= completePrecisionSrt;
  		}
  		var self =this;
  		self.async(function(){
  			self._configurePattenr();
		}, 200);
  		
     },
     
     _configurePattenr: function(){
   		if(this.isDecimal){
   			if(this.min !== null && this.min >=0 ){
   				if(this._decimalSeparator==='.'){
   					this._pattern='[0-9]*[\.]?[0-9]*' ;
   				}else{
   					this._pattern='[0-9]*[,]?[0-9]*' ;
   				}
   				
   	   		}else{
   	   			if(this._decimalSeparator==='.'){
   	   				this._pattern='-?[0-9]*[\.]?[0-9]*' ;
				}else{
					this._pattern='-?[0-9]*[,]?[0-9]*' ;
				}
   	   		}
   		}else{
   			if(this.min !== null && this.min >=0 ){
   				this._pattern='[0-9]*';
   	   		}else{
   	   			this._pattern='-?[0-9]*';
   	   		}
   		}
      },

  	/**
	 * Add hour and minute and both fields to appear as time duration.
	 * 
  	 * @param m - minutes to add to hours
	 */
  	_calculateTimeDuration : function(m){
  		var hours = this.hour;
  		var minutes = parseInt(m);
 		var hrMilliseconds = hours*60*60*1000;
 		var minMilliseconds = minutes*60*1000;
  		var duration = hrMilliseconds+minMilliseconds;
  		  		
        var min = parseInt((duration/(1000*60))%60);
        var hr = parseInt((duration/(1000*60*60)));
        if(!this.allowNegative){ // works only if negative values are not allowed.
        	  // reset duration to 0 if duration becomes negative
      		if(duration<0){
      			hr=0;
      			min=0;
      		}
      		
      		// prepend 0 to single digit minutes
    	    min = (min < 10) ? '0' + min : min;
        }
      
	    
	    // re-initialize variable, so _handleMinuteDurationValueChange
	    // can trigger when same value is entered consecutively.
	    this.minute = null; 
	    
	    // update duration field.
	    this.hour = hr;
	    this.minute = min;
	 
    	var _value= 0; 
    	if( this.minute != undefined  ){
    		_value = this.minute/60 ; 
    	}
    	if( this.hour != undefined  ){
    		_value +=  this.hour; 
    	}
    	if(this.allowNegative){
    		this.hour =(hr===0 && duration<0) ? '-' + hr : hr;
     	    this.minute = (Math.abs(min) < 10) ? '0' + Math.abs(min) : Math.abs(min);
     	    this.negativeTime=min;
    	}
    	this.value= _value;
  	},
  	
  	/**
	 * Get Duration
	 */
  	_getDuration : function(){
  		return this.hour+':'+this.minute;
  	},
  	
  	/**
	 * Update Hour - Manual Input
	 */
  	_handleHourDurationValueChange : function(){	
  		var hrElem = this.$.hourDurationElement;
  		this.hour = parseInt(hrElem.value);
  		// do not allow negative or non numbers
  		if((this.hour<0 || isNaN(this.hour)) && (!this.allowNegative)){
  			this.hour='';
  			this.hour=0;
  		}else if(isNaN(this.hour) && (this.allowNegative)){ // reset to 0 only if NaN
  			this.hour='';
  			this.hour=0;
  		}
  		else{
  			this._handleMinuteDurationValueChange();
  		}
  	},
  	
  	/**
	 * Update Minute - Manual Input
	 */
  	_handleMinuteDurationValueChange : function(){	
  		var minute = parseInt(this.$.minuteDurationElement.value);
  		// change minute value to positive when values are either -0 or less than 0
  		if((this.hour==='-0' || parseInt(this.hour)<0) && this.allowNegative){ 
  			minute=-minute;
  		}
  		if(isNaN(minute)){
  			// reset minutes if value is not a number.
  			this.minute='';
  			this.minute='00';
  		} else{
  			this._calculateTimeDuration(minute);	
  		}
  	},
  	
  	/**
	 * Update Minute/Hour
	 */
  	_updateTime: function(m){
  		if(this.value != undefined) {
  			
  			if(typeof this.value === 'string'){
  				this.hour = parseInt(this.value.split(':')[0]);
  				this.minute = parseInt(this.value.split(':')[1]);
  				this.minute = (this.minute < 10) ? '0' + this.minute : this.minute;
  				this.minute = (this.allowNegative && this.value.split(':')[0].includes('-')) ? -this.minute : this.minute;
  				this._calculateTimeDuration(this.minute);
  			}else if(typeof this.value === 'number'){
  				this.hour = parseInt(this.value);
  				var min =  (this.value - this.hour ).toFixed(2)*60;
  				min = Math.round(min);
  				if(min>0){
  					this.minute='';
  					this.minute = (min < 10) ? '0' + min : min;
  					this._calculateTimeDuration(this.minute);
  				}else{
  					if(this.allowNegative){
  						this._calculateTimeDuration(min);
  					}else{
  						this.minute='';
  	  	  	  			this.minute='00';
  	  	  	  		    this._calculateTimeDuration(0);
  					}
  				}
  			}
		}else{
		    this.value = 0;
		} 
  	}
});



/**
 * clear the field if the user set any special Character
 */
function clearSpecialCharacters (evt){
	let element = evt.target;
    var evt = (evt) ? evt : window.event;
    var ua = navigator.userAgent;
    if (evt.key == 'Dead') {
      element.value = '';
      return false;
    } else if (element.dataset.scopeId === 'valueElement') {
       if(element.value.length > 0){
          let specialCharacters = '!@#$%^&*()+=[]\\\';/{}|\":<>?` àèìòùãõâêîôûÀÈÌÒÙÂÊÎÔÛÀÕÃˆ~˜`';
            let newValue = element.value;
            let clearText = false;
            for (let index = 0; index < specialCharacters.length; index++) {
              if (newValue.indexOf(specialCharacters[index]) != -1) {
                clearText = true;
                break;
              }
            }
            if (clearText) {
              element.value = '';
              evt.preventDefault();
              return false;
            } else {
              return true;
            }
       }
    }else if(element.dataset.scopeId != 'valueElement') { 
      if (evt.type === 'keypress' || evt.type === 'keydown') {
    	if(ua.indexOf('iPhone') != -1 || ua.indexOf('iPad') != -1){
    		this.curHrCaretPos = $j(element).closest('maximo-numeric-spinner')[0].curHrCaretPos;
    	}
        var charCode = (evt.which) ? evt.which : evt.keyCode;
        var keyDownCharCode = '188,37,39'; //keycode for comma, left and right arrow
        if (evt.type === 'keydown' && keyDownCharCode.indexOf(charCode) > 1) {
          return false;
        }

        if (charCode > 31 && (charCode < 45 || charCode > 57)) {
          return false;
        }
        return true;
      } else {
        if (element.value.length > 0) {
          var curTime = element.value.match(/[0-9\.|\,\-]+/g); //Extracting the valid time value
          element.value = (curTime) ? curTime.join('') : '';
          if(this.curHrCaretPos && (ua.indexOf('iPhone') != -1 || ua.indexOf('iPad') != -1)){
  			element.setSelectionRange(this.curHrCaretPos+1, this.curHrCaretPos+1);
  		 }
          return true;
        }
      }
    } 
 } 

/**
 * check if the value set by user is valid
 */
function onlyNumberValue(evt)
{
	if(this.clearSpecialCharacters(evt)){
		var decimalNumber = 1.1;
		var decimalString = decimalNumber.toString();
		var separator ='.';
		var separatorKeyCode = $M.keyCode.PERIOD;
		if(decimalString.indexOf('.') === -1){
			separator  =  ',';
			separatorKeyCode =$M.keyCode.COMMA;
		} 	
			
		var isInteger = $j(evt.currentTarget).attr('decimal') ===  undefined;
		if(isInteger){
			return (((evt.keyCode === $M.keyCode.BACKSPACE) || (evt.keyCode ===  $M.keyCode.DELETE) )|| (evt.keyCode >= 48 &&  evt.keyCode <=57 )) // 0 1 2 3 4 5 6 7 8 9
		}else{
			if(((evt.keyCode === $M.keyCode.BACKSPACE) || (evt.keyCode===$M.keyCode.TAB) || (evt.keyCode ===  $M.keyCode.DELETE) )|| (evt.keyCode >= 48 &&  evt.keyCode <=57 )){ // 0 1 2 3 4 5 6 7 8 9
				return true;
			}else if(evt.keyCode === separatorKeyCode){ // .
				if(evt.currentTarget.value !='' &&  evt.currentTarget.value.indexOf(separator)!= -1){
					return false;
				}else{
					return true;
				}
			}
		}
		 
		return false;   
	}
	
	 
}

/**
 * check if the value set by user is valid
 */
function isNumberKey(evt)
{
	if(evt.key === '-'){ // -
		
		if(evt.currentTarget.value =='' &&  evt.currentTarget.value.indexOf('-') == -1){
			return true;
		}else if(evt.currentTarget.value.indexOf('-') == -1 && evt.currentTarget.selectionStart === 0 ){
			return true;
		}
	}
	
	return onlyNumberValue(evt);    
}


function chekInput(evt){
	  if(evt.keyCode === 8 || evt.keyCode === 37 || evt.keyCode === 38 || evt.keyCode === 39 || evt.keyCode === 40 ){
		  return true
	  }
	  let pattern =  evt.target.patternnumber;
	  var re = new RegExp(pattern);
	  let temp ;
	  let currentValue =evt.target.value;
	  if(evt.target.selectionStart===0){
	    temp = evt.key+currentValue;
	  }else if(evt.target.selectionStart===evt.target.value.length){
	    temp = currentValue+evt.key;
	  }else {
	    temp = currentValue.slice(0, evt.target.selectionStart) + evt.key + currentValue.slice(evt.target.selectionStart);
	  }

	  let result = re.exec(temp);
	  if(result===null || result === undefined){
	    return false
	  }
	  return  result[0] ===temp;
}
