/*

 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2018
 */

/*
A rich text editor component.
 */
Polymer({
    is: 'maximo-richtexteditor',
    behaviors : [ BaseComponent, HTMLEncoder ],
    properties: {
      value: {
        type: String,
        observer:'_updateValue'
      },
 
      backgroundColor: {
    	type: String,
    	value: '#ffffff'
      },
      
      color: {
      	type: String,
      	value: '#000000'
      },
      
      /**
       * Hide/Show Menubar
       */
      menubar: {
    	type: Boolean,
    	value : false
      },
      
      /**
       * Hide/Show statusbar which appears at bottom of textarea
       */
      statusbar: {
      	type: Boolean,
      	value : false
       },
      
       /**
        * Resize horizontal and Vertically
        */
      resize: {
        type: String,
        value: 'both'
      },
      
      /**
       * Text to appear in textarea describing the editable area
       */
      placeholder: {
        type: String,
        value: ''
      },
       
      /**
       * Define which buttons appear on the toolbar
       */  
      toolbar: {
          type: String,
          value: 'bold italic underline forecolor numlist bullist link'
        },
        
      /**
       * The height of the editor.
       */
      height: {
        type: String,
        value: '200px'
      },
      
      /**
       * The width of the editor.
       */
      width: {
        type: String,
        value: '100%'
      },
      
      readonly: {
    	  type: Boolean,
    	  value: false
      },
      
      props: {
    	  type: Object,
    	  observer: '_setProps'
      }, 
      
      _textareaId: {
    	  type: String
      }
    },
    
    /*
     * Remove from the Dom and destroy references
     */
    remove: function(){
    	try {
    		tinyMCE.execCommand('mceRemoveEditor', true, this.id+'_textarea');
    	}
    	catch(error){
    		//safety
    	}
    },
    
    attached: function(){
    	this.remove(); //clear it out so we can re-initialize
    	this._textareaId = this.id + '_textarea';
    	var selector = 'textarea#' + this._textareaId;
    	var rte = this;
    	rte.firstChange = true;
    	var props = {
    		mode:'textareas',
    		height: this.height,
    		width: this.width,
 	      	menubar: this.menubar,
 	      	statusbar: this.statusbar,
 	      	resize: this.resize,
 	      	selector: selector,
 	      	plugins: 'placeholder textcolor lists directionality link autolink',
 	   	    toolbar: this.toolbar,
 	   	    setup: function(editor){
            	editor.on('init',function(e) {
            		if(rte.value){
            			e.target.setContent(rte.value);
            			tinyMCE.execCommand('mceRepaint');
            		}
            		var body = $j(e.target.getBody());
            		body.css({'background-color':rte.backgroundColor,'color':rte.color});
            		var p = $j(e.target.getBody().firstElementChild);             		
            		p.css({'word-wrap':'break-word'});
            	});
            	editor.on('change', function(e){
            		tinyMCE.execCommand('mceRepaint');
            		//the following line is to fix a problem where tinymce throws an onchange after first character is typed
            		if(!rte.firstChange || (document.activeElement !== e.target.getContentAreaContainer().firstElementChild)){
            			rte.value = e.target.getContent();
            			if(!rte.value || rte.value === ''){
            				tinyMCE.execCommand('mceRepaint');
            			}
            		}else if(e.target.getContent() === '' || rte.value !== e.target.getContent() ){ //Will handle if user remove some or all content by ctrl X
            			rte.value = e.target.getContent();
            		}
            		var p = $j(e.target.getBody().firstElementChild);
            		p.css({'word-wrap':'break-word'});
            		rte.firstChange = false;
            	});
            	//The following event will handle copy/paste content into editor
            	editor.on('PastePostProcess', function (e) {
            		  rte.value += e.target.getContent(); 
            	});
            	editor.on('keydown',function(e) {
            		tinyMCE.execCommand('mceRepaint');
            		if (e.target){
	            		var element = $j(tinyMCE.activeEditor.getBody().firstElementChild);           		
	            		element.css({'word-wrap':'break-word'});
            		}
            	});
				editor.on('keyup',function(e) {
            		tinyMCE.execCommand('mceRepaint');
            		if (e.target){
            			rte.value = tinyMCE.activeEditor.getContent(); 
	            		var element = $j(tinyMCE.activeEditor.getBody().firstElementChild);           		
	            		element.css({'word-wrap':'break-word'});
            		}
            	});
 	   	    }
	 	};
	 	if(this.readonly){
	 		props.readonly = 1;
	 		props.menubar = false;
	        props.statusbar = false;
	        props.toolbar = false;
	        rte.placeholder = this.localize('uitext','mxapibase','ReadonlyEmptyLongDesc');
	 	}
	 	tinyMCE.init(props);
		this.initialized = true;
    },
    
    _updateValue: function(newValue){
    	if(this.initialized){
    		tinyMCE.execCommand('mceRepaint');
    	}
    },
    
    _setProps: function(newProps){
    	var rte = this;
    	Object.keys(newProps).forEach(function(prop){
    		rte.setAttribute(prop, HTMLEncoder.sanitize(newProps[prop]));
    	});
    },
    
    forceClear: function(){
    	var textArea = tinyMCE.get(this.id+'_textarea');
    	
    	var self = this;
		this.async(function(){
			self.value='';
			if(textArea &&   ( typeof textArea.setContent  === 'function') ){
				textArea.setContent('');
	    	    tinyMCE.execCommand('mceRepaint');
	    		//textArea.render();
	    	}
		}, 200);
		
    	
    },
    
    getValue: function(){
    	return tinyMCE.get(this.id+'_textarea').getContent();
    },
    
    updateContent: function(html){
    	var textArea = tinyMCE.get(this.id+'_textarea');
    	if(textArea){
    		textArea.setContent('');
    		tinyMCE.execCommand('mceRepaint');
    		this.value = html;
    		textArea.setContent(html);
    	}else{
    		this.value = html;
    	}
    },
    
});
