/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2018
 */
 /*
`<maximo-table>` element. A table display of data.

Example:
```html
	<maximo-table id="tableid" collection="{{getLocalObject('collection_short')}}" selected-index="[0]" summary="demo table 1"
	columns='[{"attribute":"first_name","title":"First Name","flexGrow":"1.5"},{"attribute":"last_name"},{"attribute":"first_name","title":"First Name"},
	{"attribute":"last_name"},{"attribute":"first_name","title":"First Name"},{"attribute":"last_name"},{"attribute":"first_name","title":"First Name"},
	{"attribute":"last_name"}]'></maximo-table>
```
	Column definition for having checkbox - E.g. 
	{'attribute':'status','title':self.jobplanwoschema.properties.status.title,'sortable':true,'checkbox':true,'listener':'update-status','internalattribute':'status_maxvalue','checkboxchecked':'ACTIVE','checkboxlabel':'Active','checkboxlabelfalse':'Inactive','additionalattribute':'DRAFT','additionalattributelocalized':'Draft'}


### Accessibility
&#8593; or &#8592; Choose previous radio button(wraps to beginning)<br>
&#8595; or &#8594; Choose next radio button(wraps to end)

@demo

*/

Polymer({
	is: 'maximo-table',
  	behaviors: [BaseComponent, ArrowKeys, DragScrollComponent, InfiniteScroll, Hierarchical,Multiselect, Tabledetail],
  	listeners: {
  		'collection-data-refresh-requested':'_collectionRefreshRequested',
  		'collection-data-refreshed':'_collectionDataRefreshed',
		'selectAll': '_selectAll',
		'maximo-qrcode-result' : '_returnQRcode'
  	},
  	properties : {
		/** Column definitions passed as a property. This will take precedence over internal column definitions.  */
		columns: {
			type: Array,
			value: function(){
				return [];
			},
			notify:true
		},
		/** data to display */
		items: {
			type: Array,
			value: function(){
				return [];
			},
			observer: '_itemsChanged'
		},
		/** Collection to use for display data */
		collection: {
			type: Object,
			observer: '_setCollection'
		},
		/**
		Conditional Action call-back function.
		*/
		actionCallback: {
			type: Object
		},
		/** Summary to set on the table. This will give a summary of what this table shows and is required for accessibility. */
		summary: {
			type: String
		},
		/** Height to set on the table body */
		maxHeight: {
			type: String,
			value: '300px'
		},
		/** Width to set on the table */
		width: {
			type: String,
			value: 'auto'
		},
		whiteSpace: {
			type: String,
			value: 'nowrap'
		},
		/** Column definitions to use when building */
		_displayColumns: {
			type: Array,
			computed: '_getMyColumns(columns)'
		},
		_loading: {
			type: Boolean,
			value: false
		},
		_noRecords: {
			type: Boolean,
			value: false
		},
		/** Index of selected record */
		selectedIndex: {
			type: Array,
			value: function(){
				return [];
			}
		},
		/** Select mode ('single' or 'multiple') */
		selectMode: {
			type: String,
			value: 'none'
		},
		_arrowKeyInfo: {
			type: Object,
			value: function(){
				return {'selector':'tr'};
			}
		},
		_ready: {
			type: Boolean,
			value: false
		},
		/**
			This is the object used to populate the actions column.
			header:{
				width: width as px e.g. 500px
			}
			list:[
				{
					'icon': icon-path // optional, will default to button if no path is specified (icons:arrow-drop-up)
					label: label text
					'action': actionName//required, id for this action. This will be how we figure out what was clicked when listening for the event.
					''
				}
			]

			if there is no icon, then a button will be generated with the selected label
		**/
		actions:{
			type: Object,
			value: function(){
				return {};
			}
		},

		/*
			the attribute with which to order the collection by. Will notify the parent property.
		*/
		collectionOrderBy:{
			type: String,
			notify: true
		},

		maxWidth:{
			type: String,
			notify:true
			
		},
		/**
		 * white-space property for table header
		 */
		whiteSpaceForHeader: {
			type: String,
			value: 'nowrap'
		},
		/**
		add dynamic class to table.
		*/
		addClass:{
			type: String
		},
		
		/**
		 * indicate if the collection is loaded
		 */
		_isCollectionLoaded: {
			type: Boolean,
			value: false
		},
		noRecordsLabel:{
			type:String,
			value: ''
		},
		multiSelectCheckbox: {
			type: Boolean,
			value: false
		},
		selectedRecords:{
			type: Array,
			value: function(){
				return [];
			}
		},
		allRowSelected:{
			type:Boolean,
			value:false
		},
		/**support sorting on multiple attributes*/
		multisort: {
			type: Boolean,
			value: false
		},
		/**Array to store multiple sorting attributes for orderBy string */
		multisortAttributes: {
			type: Array,
			value: function(){
				return [];
			}
		},
		/**Number of attributes allowed for multiple sorting*/
		multisortLength:{
			type: String,
			value: '3'
		},
		/**Enable/disable filter*/
		filter: {
			type: Boolean,
			value: false
		},
		/**Show/hide filter row*/
		filterRow: {
			type: Boolean,
			value: false
		},
		numericspinnercolnum:{
			type: String,
			value: ''
		},
		numericspinnerminval:{
			type: String,
			value: ''
		},
		isrefreshCollection: {
			type: Boolean,
			value: true
		},
		selectedColumn :{
			type: String,
			value: ''	
		},
		isDecimal: {
			type: Boolean,
			value: false
		},
		serialnumber : {
			type: Boolean,
			value: false
		},
		/**Enable/disable scanner*/
		scanner: {
			type: Boolean,
			value: false
		},
		headerActions:{
            type: Object,
            value: function(){
                return {};
            }
        },
	},

	/**
		Method for determining whether or not we have actions to display
	**/
	_hasActions: function(actions){
		//TODO: check if null
		return !$j.isEmptyObject(actions) && (actions.list && !$j.isEmptyObject(actions.list));
	},
	/**
		We indicate if an action has an icon through the presence of the icon property
	**/
	_actionHasIcon: function(action){
		return action.icon ? true : false;
	},
	_getMyColumns: function(columns){
		return this.columns && this.columns.length>0?this.columns:this.childColumns;
	},
	_getTitle: function(record,datacolumn){
		var value = $M.getRelatedData(record, datacolumn.attribute);
		if(value){
			return value;
		}
		return '';
	},
	
	/**
	*  Check if the value matches 
	*/
	_getCheckBoxFlag: function(record,datacolumn){
		var columnData;
		if(datacolumn.internalattribute) {
			columnData = $M.getRelatedData(record, datacolumn.internalattribute);
		}else {
			columnData = $M.getRelatedData(record, datacolumn.attribute);
		}
		if(columnData && columnData===datacolumn.checkboxchecked){
           return true;
        }
		return false;
	},
		
	/**
	 * Get the false label for checkbox.
	*/
	_getCheckboxLabelFalse: function(record,datacolumn){
		var label = datacolumn.checkboxlabelfalse;
		var columnData=$M.getRelatedData(record, datacolumn.attribute);
		if(datacolumn.additionalattribute && columnData===datacolumn.additionalattribute){
			label = datacolumn.additionalattributelocalized;
		 }
		return label;
	},
	
	_getData: function(record,datacolumn,trunc){
		var columnData=$M.getRelatedData(record, datacolumn.attribute);
		if(datacolumn.columndata && columnData===''){
			columnData = datacolumn.columndata;
		}
		if(datacolumn.alternateattribute && columnData===''){
		    columnData= $M.getRelatedData(record, datacolumn.alternateattribute);
		 }
		if(datacolumn.type && datacolumn.type==='quantity'){
			if(Number.isInteger(parseFloat(columnData.replace(/[^0-9 | ^. |^-]/g, '')))){
				columnData = parseInt(parseFloat(columnData.replace(/[^-0-9 | ^.]/g, '')));
			} 			
		}
		if(datacolumn.type && datacolumn.type==='date' && datacolumn.format && columnData){
			if(moment(columnData).isValid()){
				columnData = moment(columnData, datacolumn.format).format(datacolumn.format);
			} else if(moment(columnData, 'YYYY-MM-DDThh:mm:ss').isValid()){
				columnData = moment(columnData, 'YYYY-MM-DDThh:mm:ss').format(datacolumn.format);
			}
		}
		if(datacolumn.numericspinnerattribute && columnData===''){
			columnData= $M.getRelatedData(record, datacolumn.numericspinnerattribute);
			if(columnData === '') {
				return;
			}
		}
		return columnData;
	},
	
	_getMaxValue: function(record,datacolumn){
		var columnData= '';
		if(datacolumn.maxvalue){
			columnData= $M.getRelatedData(record, datacolumn.maxvalue);
		 }
		if(!columnData){
			return datacolumn.maxvalue;
		}
		return columnData;
	},
	
	_hasImageData: function(record,datacolumn,trunc){
		return (datacolumn.image) ? true  : false ;
	},
	_getImageData: function(record,datacolumn,trunc){
		var columnData=$M.getRelatedData(record, datacolumn.image);
		return columnData;
	},
	
	_getColumnTitle: function(column){
		return column.headertitle ? column.headertitle : (column.title ? column.title : '');
	},
	_getColWidth: function(column,index){
		if(column.width){
			return column.width;
		}
		var length = this._displayColumns.length;
		if(this._hasActions(this.actions)){
			length++;
		}
		return 100/length+'%';
	},
	_keyHandler: function(e){
		this._rowTapHandler(e);
	},
	_setSelected: function(indexes){
		if(!this._selectable()){
			this.selectMode = indexes.length===1?'single':'multiple';
		}
	},
	_rowTapHandler: function(e){
		if(this._showMore(e)){
			return;
		}
		if(!this._selectable() || this.multiSelectCheckbox){
			return;
		}
		var ctrlKey = e.detail?e.detail.sourceEvent.ctrlKey:e.ctrlKey;
		var shiftKey = e.detail?e.detail.sourceEvent.shiftKey:e.shiftKey;
		var last;
		if(this.selectedIndex.length>0){
			last = this.selectedIndex[this.selectedIndex.length-1];
		}
		if(last>=0 && this.selectMode === 'multiple' && (ctrlKey || shiftKey)){
			var selected = $j(e.currentTarget).attr('aria-selected') === 'true';
			var index = e.model.index;
			if(ctrlKey){
				if(selected){
					$M.arrayRemove(this.selectedIndex,index);
				}
				else {
					this._selectRecord(index);
				}
			}
			else if(shiftKey){
				var move = index-last;
				var start = move>0?last:index;
				move = Math.abs(move)+start;
				this.selectedIndex = [];
				for(var addIndex = start; addIndex<=move;addIndex++){
					this._selectRecord(addIndex);
				}
			}
		}
				
		else { //single select
			this.selectedIndex = [];
			this._selectRecord(e.model.index);
		}
		this.selectedColumn= $j(e.target.closest('.tablecol')).length? $j(e.target.closest('.tablecol'))[0].dataArgs : '';
		this._selectRows(this.selectedIndex, true);
	},
	_selectRecord: function(index){
		if(this.collection){
			this.collection.selectedRecordIndex = index;
		}
		this.selectedIndex.push(index);
	},
  	created : function(){
  		this.childColumns = [];
		var table = this;
		$j(this.childNodes).each(function(){
			var child = $j(this)[0];
			if(child.tagName==='MAXIMO-TABLECOL'){
				var col = {};
				for (var index = 0; index < child.attributes.length; index++){
					var item = child.attributes.item(index);
					col[item.name] = item.value;
				}
				table.childColumns.push(col);
			}
		});
  	},
		/**
			Checks the column metadata for a sortable property. If it exists, use the boolean from that. Otherwise, we set this column
			to not sortable
		**/
		_isColumnSortable: function(sortable){
			return sortable ? sortable : false;
		},
		/**
			Called when a table header is clicked. The column information was stored in data-args as an attribute.
			Fires an event with the column information so we are able to call the data from the server again, this time
			with a different sorting.
		**/
		_headerClicked: function(e){
			if(this.selectedRecords && this.selectedRecords.length)
			{
				return;
			}
			else{
				e.stopPropagation();
				//if the data is based on a collection and we are using the orderby parameter
				var dataArgs = $j(e.target.closest('.tableheader')).length? $j(e.target.closest('.tableheader'))[0].dataArgs : '';
				if(this.collectionOrderBy !== undefined){
					this.collectionSort(dataArgs);
				}
				else{
					//otherwise, let the calling code handle it for us. It could either be from a resource or
					//a client side operation, in which case we must manually sort.
					this.fire('table-header-clicked',dataArgs);
					}
			}
		},
		/**
			Called when rendering each table header item. Will change the filter icon based on the column metadata.
			ascending
			descending
			null - neutral
		**/
		sortType: function(column, displayColumns){
		 if(this._isColumnSortable(column.sortable)){
			if(column.sortType === 'ascending'){
				return 'Maximo:Sort-Ascending';
			}else if (column.sortType === 'descending'){
				return 'Maximo:Sort-Descending';
			}else{
				return 'Maximo:Sort';
			}
		 }else{
			 return '';
		 }
		},
		/**
			Retrieve the default next sort type. The order is:
			1. Neutral
			2. Ascending
			3. Descending

			Once we are at descending, we loop back to neutral.
		**/
		nextSortType: function(column){
			switch(column.sortType){
				case 'neutral':
					return 'ascending';
				case 'ascending':
					return 'descending';
				case 'descending':
					return 'neutral';
				default:
					//we assume no sort type is neutral, so the next step would be ascending.
					return 'ascending';
			}
		},

		/**
			If the column is sortable, the cursor should become a pointer when hovering over
		**/
		_isCursorPointer: function(column){
			return column.sortable ? 'cursor: pointer' : '';
		},

		/**
			Fires an event indicating that an action was done. The parent element can handle it however they want.
		**/
		tableActionClicked: function(e){
			var toSend = {
				action: e.model.action,
				record: e.model.record,
				source: e.currentTarget,
				rowNum: this._getRow(e.target)
			};
			return this.fire('table-action-clicked',toSend);
		},
		
		
		/**  
	  	 * 
		 * return the action 
		 *
		 */
	  	_getRow : function( element){
	  		 'use strict';
	  		if( element.hasAttribute('is-row')){
	  			return Number(element.getAttribute('data-row-num'));
	  		}
	  	 
	  		return this._getRow(element.parentElement);
	  	},

  	ready : function(){
  		this._ready = true;
  		this.dragScrollElement = this.$.body;
  		var self = this;
  		$j(window).on('resize', function(){
  			if(self.headerFixed){
  				this.scrollTo(0,0);
  			}
  			self._toggleFixed(self.headerFixed, self.headerHeight);
		});
  	},
  	attached: function(){
  		var ironLists = $j(this.$.tbody).find('iron-list');
		if(ironLists.length === 1){
			ironLists[0].scrollTarget = this.$.tbody;
			this.infiniteScrollingList(ironLists[0]);
			
		}
		if(this.height==='auto'){
			$j(this.$.body).css({'overflow-y':'auto'});
		}
		if(!this.collection){
			this._collectionDataRefreshed();
		}
		var tableComponent = this;
		if(this.width === '100%'){
			$j(window).on('resize', function(){
				tableComponent._updateColumns(); 
				window.clearTimeout(tableComponent.resizeTimer);
				tableComponent.resizeTimer = setTimeout(function(){
					tableComponent._sizeTable();
				}, 150);
			});
		}
		this.async(function(){
			tableComponent._updateColumns();
		}, 500);
		
		if(this.filter) {
			this.filterRow = true;			
		} else {
			this.filterRow = false;
		}		
		if(!this.summary){
  			console.error('maximo-table '+this.id+ ' requires a summary attribute that describes this table.');
  		}
  	},
  	
  	/** update columns width **/
  	_updateColumns: function(){
  		var tableComponent = this;
  		$j(tableComponent.$.body).find('tr:not(.stopHover)').each(function(){
			$j(this).find('td').each(function(index){
				$j(this).find('div').each(function(){ 
					var th = null;
					if(tableComponent.columns[index]){
						th = tableComponent.querySelector('#tableheader_'+ tableComponent.columns[index].attribute); 
					}
					
					if(th){
						var width = th.style.width;
						if(width !== undefined && width!==''){
							if(width.indexOf('%') !== -1 ){
								if(tableComponent.offsetParent && tableComponent.offsetParent.offsetWidth){
									width = (parseInt(width.replace('%', '')) / 100) * tableComponent.offsetParent.offsetWidth  ;
								}else{
									width = (parseInt(width.replace('%', '')) / 100) * window.innerWidth  ;
								}
							} 
							$j(this).css('width' ,  width   );
							$j(this).css('max-width', width  );
						}else{
							var length = tableComponent._displayColumns.length;
							if(tableComponent._hasActions(tableComponent.actions)){
								length++;
							}
							
							if(tableComponent.offsetParent.offsetWidth){
								width = ((100/length)/100) * tableComponent.offsetParent.offsetWidth  ;
							}else{
								width = ((100/length)/100) * window.innerWidth  ;
							}
							
							$j(this).css('width' ,  width   );
							$j(this).css('max-width', width  );
						}
					}
				});
			});
		});
  	},

	_getRowRole: function(){
		if(this.hierarchical){
			return 'treeitem';
		}
		return;
	},
	
  	_tabIndex: function(index){
  		return this._isSelected(index)?'0':'-1';
  	},
  	_isSelected: function(index){
  		return (this._selectable() && $M.arrayContains(this.selectedIndex,index))?'true':'false';
  	},
  	_selectable: function(){
  		return this.selectMode === 'single' || this.selectMode === 'multiple';
  	},
  	_selectRows: function(indexes, multi){
		if(!this._selectable()){
			return;
		}
  		var rows = $j(this.$.tbody).find('.tablerow');
  		rows.removeAttr('aria-selected');
  		rows.attr({'tabindex':'-1'});
  		indexes.forEach(function(index){
  	  		var selectedRow = rows[index];
  			if(selectedRow){
  				$j(selectedRow).attr({'aria-selected':'true','tabindex':'0'});
  			}
  		});
  		this.fire('maximo-table-rows-selected', this);
  	},
  	_sizeTable: function(){
  		return;
		var tableComponent = this;
		var headerWidths = [];
		var total = 0;
		if(this.width === '100%'){
			$j(this.$.header).css('width','100px');
		}
		
		$j(tableComponent.$.body).find('tr:not(.stopHover)').each(function(){
			$j(this).find('td').each(function(index){
				$j(this).find('div').css({'width':'auto'});
				$j(this).css({'width':'auto'});
			});
		});
		
		this.async(function(){
			$j(this.$.header).css('width','100%');
			$j(this.$.header).find('th').each(function(){
				var headerWidth = this.offsetWidth;
				//console.log(headerWidth);
				if($j(this).text().trim()===''){
					headerWidth = 50;
				}
				headerWidths.push(headerWidth);
				total+= headerWidth;
				//TODO: Why are we doing this set? We have defined width already through a function. By assigning it this way,
				//we effectively ignore our property.
				//$j(this).width($j(this).width());
			});
			//$j(this.$.header).width(total);
			if(this.$.body.scrollHeight > this.$.body.offsetHeight){
				total = total + getScrollbarWidth();
			}
			$j(this.$.body).width(total);
	
			$j(tableComponent.$.body).find('tr:not(.stopHover)').each(function(){
				$j(this).find('td').each(function(index){
					var width = headerWidths[index];
					$j(this).find('div').css({'width':width+'px'});
					$j(this).css({'width':width+'px'});
				});
			});
	
			$j(this.$.wrapper).css({'max-width':$j(this.$.wrapper).parent().width()+'px'});
		});
  	},
  	_itemsChanged: function(){
  		if(!this._ready){
  			return;
  		}

		//we clear the selected rows when a new set of items is found
		this._selectRows([], true);
		//we need to clear the selected index as well
		this.set('selectedIndex', []);
		if(this.items !== undefined && this.items.length===0){
			 this._noRecords = false;
			 this._loading = true;
			 var selectAll=$j('#allSelect');
			 if (this.multiSelectCheckbox && selectAll.length>0){
				 selectAll[0].hidden=true;
			 }
			 if(this.collection === undefined || !this.collection.collectionData || this.collection.collectionData.length === 0){
				 this.async(function(){
					 if(this.items !== undefined && this.items.length===0){
						 this._noRecords = true;
						 this._loading = false;
					 }
				 },1000);
			 }
		}
		if(this.items !== undefined && this.items.length>0){
		
		// notifyResize if table is only using the items array without Referencing collection.
			var ironlist = $j(this).find('iron-list');
			if(ironlist && ironlist.length>0 && !this.collection){
				setTimeout(function(){ 
	               $j(ironlist)[0].notifyResize();
				},500);  
			}
			this._noRecords = false;
			this._loading = false;
			 var selectAll=$j('#allSelect');
			 if (this.multiSelectCheckbox && selectAll.length>0){
				 selectAll[0].hidden=false;
			 }
	 		var tableComponent = this;
	 		this.async(function(){
	 			this._sizeTable();
	 		},400);
		}
  	},
  	_collectionDataRefreshed: function(e){
  		if(this.collection && e.detail===this.collection){
			var schema = this.collection.get('schema');
				if(schema){
				var table = this;
				$j(this.$.thead).find('.tableheaderrow').each(function(index){
					if($j(this).text().trim()===''){
						var tableDisplayCol=table._displayColumns[index];
						if (tableDisplayCol){
							var attribute = tableDisplayCol.attribute;
							if(schema.properties && schema.properties[attribute]){
								var property = schema.properties[attribute];
								$j(this).text(property.title);
							}
						}
					}
				});
			}
	  		this.items = this.collection.collectionData;
	  		this._setSelected([this.collection.selectedRecordIndex]);
	  		var tableComponent = this;
	  		this.async(function(){
	  			tableComponent._isCollectionLoaded = true;
	  			tableComponent._setSortType();
			}, 1000);
  		}
  	},
  	_setCollection: function(collection){
  		if(this.selectMode==='multiple'){
  			this.selectMode='single';
  		}
  		collection.addComponentListener(this, 'collection-data-refresh-requested');
  		collection.addComponentListener(this, 'collection-data-refreshed');
  	},
  	getSelected: function(){
  		if(this.collection){
  			return this.collection.selectedRecord;
  		}
  		var temp = [];
  		var items = this.items;
  		this.selectedIndex.forEach(function(index){
  			temp.push(items[index]);
  		});  		
  		return this.selectedRecords.length>0?this.selectedRecords:temp;
  	},
  	/**Get index of sorting attribute in the array*/
  	_getIndexOfAttribute(attribute) {
  		return this.multisortAttributes.indexOf('-' + attribute) > -1 ? this.multisortAttributes.indexOf('-' + attribute) : this.multisortAttributes.indexOf('%2B' + attribute);
  	},

		//common sort case specifically done through the collection itself
		/**
			Typical usage is to call table.collectionSort in the event handler
		**/
		collectionSort: function(clickedColumn){
			//determine the next sort type
			if(!clickedColumn.sortable){
				return;
			}
			
			//sortattribute added in case we need sorting on different attribute than display attribute 	
			var attribute = clickedColumn.sortattribute ? clickedColumn.sortattribute : clickedColumn.attribute;

			//store the next sort type
			var nextSortType;
			var self = this;

			var toSet = [];

			//update the columns and reset all sort types except the chosen one. The chosen one progresses to the next sort type.
			this.columns.forEach(function(col){

				//if the clicked column attribute is the same as the current attribute in the loop
				if(clickedColumn.attribute === col.attribute){
					//change to the next sort type.
					col.sortType = self.nextSortType(col);
					nextSortType = col.sortType;
				}else if(!self.multisort){ //don't set sotType to neutral in case of multiple sorting
					//otherwise, we set the sort type to neutral. All other columns besides the selected one are reset
					col.sortType = 'neutral';
				}
				//allow table to detect which sort type should be next
				toSet.push(col);
			});
			//Remove the element from orderBy if already exist because it can be ascending/decending and we want to sort on different sortType 
			var index = this._getIndexOfAttribute(attribute);
			if(index > -1) {
				this.multisortAttributes.splice(index, 1);
			}
			//Remove the first element from orderBy if number of attributes have reached to specified multisortLength 
			if(this.multisortAttributes.length === parseInt(this.multisortLength)){
				this.multisortAttributes.splice(0, 1);
			}
			//when descending, the attribute type must have a - in front for a standard maximo collection
			if(nextSortType === 'descending'){
				if(this.multisort){
					//In case of multisort push the attribute to array
					this.multisortAttributes.push('-' + attribute);
					attribute = this.multisortAttributes.toString();
				} else {
					attribute = '-' + attribute;
				}
			}
			//when ascending, the attribute type must have a + in front for a standard maximo collection
			else if(nextSortType === 'ascending'){
				if(this.multisort){
					this.multisortAttributes.push('%2B' + attribute);
					attribute = this.multisortAttributes.toString();
				} else {
					attribute = '%2B' + attribute;
				}
			}
			//when neutral, we don't need anything
			else if(nextSortType === 'neutral'){
				if(this.multisort){
					attribute = this.multisortAttributes.toString();
				} else {
					//remove attribute
					attribute = '';
				}
			}
			
			//we set the sort type to neutral for all attributes which are not in orderBy list
			if(this.multisort){
				this.columns.forEach(function(col){
					var attribute = col.sortattribute ? col.sortattribute : col.attribute;
					if(self._getIndexOfAttribute(attribute) === -1){
						col.sortType = 'neutral';	
					}
				});
			}
			
			//first set the search query, then set the columns
			this.set('collectionOrderBy',attribute);
			//now we set the columns and the search query and return the new search query. The search query must be
			this.set('columns',toSet);
			this.fire('maximo-table-collection-sort');
		},
		_getActionLabel: function(action, record){
			if(action){
				if(action.labelattribute !== undefined && record !== undefined){
					return record[action.labelattribute];
				}else{
					return action.label;
				}
			}
		},
		_details : function(e){			
			if(e.model.__data__.record && e.model.__data__.record._moredata_){
				return;
			}
			this.fire(e.model.__data__.datacolumn.listener,e);						
		},
		_cellClass: function(record,datacolumn){
			var data = this._getData(record,datacolumn);
			if(data===$M.getNoDataString()){
				return 'notEntered';
			}
			return '';
		},
		
		/**
		 * To check if this template is for hyperlink or checkbox or others 
		 */
		_defaultTypeCheck : function(record,datacolumn){
			if(this.isHyperlinkEnabledAndColumnDataExists(record,datacolumn) || datacolumn.checkbox || datacolumn.numericspinnerattribute || this._showMaximoText(datacolumn)){
				return true;
			}
			return false;
		},
		
		/**
		 * Get Custom Style based on the value if specified,  Add Check Attribute 
		 * E.g  usage of conditional formatting in table's column definition 
		 * 'conditionalformatting':'{"attribute":"status_maxvalue","values": {"NEW": "color:#d74108","INPROG": "color:#be9b00"}}' 
		 */
		_getCustomeStyle: function(record,datacolumn){
            if(datacolumn.conditionalformatting){
                var styleObject = JSON.parse(datacolumn.conditionalformatting);
                var columnData;               
                if(styleObject.customcondition === 'true'){
                    this.fire('custom-conditional-formatting',{'record':record,'datacolumn':datacolumn});
                    return;
                }
                if(styleObject.attribute){
                    columnData=$M.getRelatedData(record, styleObject.attribute);
                }
                if(!columnData){
                    columnData=$M.getRelatedData(record, datacolumn.attribute);       
                }               
                var style = styleObject.values[columnData];
                if(style){
                    return style;
                }
            }
            return '';
        },
        
    	_getColumnImgStyle: function(record,datacolumn){
            if(this._hasImageData(record,datacolumn)){
                return 'display:flex;';
            }
            return '';
        },
		/**
		function to decide if table has action callback then  callback function
		will be called.
		*/
		_isActionVisible: function(record,action){
			if (this.actionCallback){
				return this.actionCallback.apply(this, [record,action]);
			}else{
				return true;
			}
		},
		/**
		 * Function to check if hyperlink for the column has data or not
		 * if hyperlink is enabled and data is blank, there would not be link to it.
		 */		
		isHyperlinkEnabledAndColumnDataExists: function(record,datacolumn){
			var columnData=$M.getRelatedData(record, datacolumn.attribute);
			if(datacolumn.alternateattribute && columnData===''){
			    columnData= $M.getRelatedData(record, datacolumn.alternateattribute);
			}
			
			if(datacolumn.hyperlink && columnData!==''){
				if(datacolumn.hyperlinkCondition){
					return eval(datacolumn.hyperlinkCondition);
				} 
				return true;
			}else {
				return false;
			}
		},
			/**
		 * Return the column width based on availability of sort type for the column
		 **/
		_getColumnInternalWidth: function(column){
		 if(this._isColumnSortable(column.sortable)){
			 return '90';
		 }else{
			 return '100';
		 }
		},
	  	/**
	  	 * Notify Polymer of child objects that changed.  Polymer then can handle the refresh.
	  	 */
		_notifyPathForPanel: function(notifyString){
			var children = this.$.rowRepeat.$.items.children;	
			var self=this;
			Array.prototype.slice.call(children).forEach(function(child){
				if(child.tagName !== 'TEMPLATE'){
					self.recordSelected(notifyString);
				}
			});
		},
		/**
	  	 * Does this panel support Muli-select mode
	  	 * returns true if the card has a maximo-selectall defined
	  	 */
	  	isMultiSelectEnabled: function(){
			return this.multiSelectCheckbox;
	  	},
		_getRowId: function(record){
			if(record._id){
				return this.id+'_tablerow_'+record._id;
			}
			return '';
		},
		_whetherSlider : function(datacolumn) {
			return datacolumn.noslider ? false : true;
		},
		/**
	  	 * Get the sorting order of the clicked column	  	 
	  	 */
		_getSortOrder: function(column, displayColumns){
			if(!column.sortable || !this.multisort){
				return;
			}
			var attribute = column.sortattribute ? column.sortattribute : column.attribute;
			if(this.multisortAttributes.length){
				var sortOrder = this._getIndexOfAttribute(attribute);
				if(sortOrder > -1) {
					return sortOrder + 1;
				}
			}
		},
		_hasDetailData: function(column){
			return column === 0 && !$j.isEmptyObject(this.detailAttributes);
		},		
		_hasinsertTableDetailRow: function(rec){
			return !rec._tabledetail;
		},
		/**
	  	 * Show/hide filter row  	 
	  	 */
		_toggleFilter: function() {
			if(this.filterRow) {
				this.filterRow = false;
				this._clearFilter();
			} else {
				this.filterRow = true;
				this.fire('filterToggled');
			}
			if($j(window).scrollTop()){
				window.scrollTo(0,0);
			}
		},
		/**
	  	 * Set filterValue input	  	 
	  	 */
		_filterInput: function(e){
			var column = this.querySelector('#tableFilters').itemForElement(e.target);
			column.filterValue = e.target.value;			
		},
		/**
	  	 * Refresh collection
	  	 */
		_refreshCollection: function(){
			var self = this;
			this.collection.totalCount = -1;
			self._loading = true;
			this.collection.refreshRecords().then(function(){
				self._loading = false;
			}.bind(this), function(error) {
				self._loading = false;
			});
		},
		/**
	  	 * Apply filter on click of confirm button
	  	 */
		_applyFilter: function(){
			var filterData = [];
			var self = this;
			this._displayColumns.forEach(function(column, index){
				if((column.attribute || column.numericspinnerattribute) && column.filterValue){
					//need to apply this column to a filter
					filterData.push({
							'filtertype': 'SIMPLE',
							'field': column.filterattribute ? column.filterattribute : column.attribute,
							'schemafield': column.attribute,
							'fieldtype': column.type,
							'fieldformat': column.format,
							'availablevalues': [ {
								'value': column.filterValue,
								'description': column.title,
								'selected': true }
						]
					});
				}
			});
			if(this.collection){
				this.collection.set('filterData',filterData);
				if(this.isrefreshCollection){
					this._refreshCollection();
				}
				this.fire('maximo-table-filter-data');
			}
		},
		/**
	  	 * Apply filter on Enter keypress
	  	 */
		_filterKey: function(e){
			if(e.keyCode===$M.keyCode.ENTER){
				this._applyFilter();
			}
		},
		/**
	  	 * Returns true only when filter is enabled and no actions defined
	  	 */
		_hasFilterNoActions: function(actions){
			return (this.filter && !this._hasActions(actions));
		},
		/**
	  	 * Clear filter on click of reset button
	  	 */
		_clearFilter: function(){
			$j(this.querySelector('#tableFilterRow')).find('maximo-text').each(function(){
				$j(this).val('');
			});
			this._displayColumns.forEach(function(column, index){
				if(column.filterValue){
					delete column.filterValue;
				}				
			});
			this.async(function(){
				this._applyFilter();			
			});
		},
		/**
	  	 * Set sortyType of columns to display icon as per collection sorting
	  	 */
		_setSortType: function() {
			var self = this;
			var toSet = [];
			this._displayColumns.forEach(function(col){
				var attribute = col.sortattribute ? col.sortattribute : col.attribute;
				if(self.collectionOrderBy) {
					var orderbyArr = self.collectionOrderBy.split(',');
					if($j.inArray('%2B'+attribute, orderbyArr) > -1){
						col.sortType = 'ascending';	
					} else if($j.inArray('-'+attribute, orderbyArr) > -1) {
						col.sortType = 'descending';	
					} else {
						col.sortType = 'neutral';
					}
				}
				toSet.push(col);				
			});
			this.set('columns',toSet);
		},
		_changeStyle: function(value){
			if(value===$M.getNoDataString()){
				return 'notEntered';
			}else{
				return 'detailvalue';
			}
		},
		/**
	  	 * _geneatUniqueId function return the unique id,
	  	 * it's remove whitespace from string and append with label value.
	  	 */
		_geneatUniqueId: function(value){
			return value.replace(/\s/g, '_');
		},
		
		/** Toggle the fixed header*/
	  	_toggleFixed: function(fixed, height){
			var thead = $j(this.$.thead);
			var tbody = $j(this.$.tbody);
			var filterRow = $j(this).find('#tableFilterRow');
			if(height === undefined){
				height = 0;
			} 
			if(fixed === undefined){
				fixed = false;
			}
			this.headerFixed = fixed;
	  		this.headerHeight = height;
			var filterTop = height + thead.height();
			var pos = this.parentElement.getBoundingClientRect();
			thead.toggleClass('tablefixed',fixed);
			filterRow.toggleClass('tablefixed',fixed);
			thead.css({'top':(fixed ? height+'px' : 'auto')});
			filterRow.css({'top':(fixed ? filterTop+'px' : 'auto')});
			var calcWidth = 0;
			var scrollBarWidth = 0;
			if(tbody[0] && (tbody[0].scrollHeight > tbody[0].clientHeight)){
				scrollBarWidth = window.scrollbarWidth;
			}
			if(pos.width){
				calcWidth = (window.outerWidth - pos.width) + scrollBarWidth;
			}
			if(fixed && calcWidth !== undefined){
				thead.css({'width':'calc(100vw - '+(calcWidth)+'px)'});
				filterRow.css({'width':'calc(100vw - '+(calcWidth)+'px)'});
			} else {
				thead.css({'width':'auto'});
				filterRow.css({'width':'auto'});
			}
			var wcCalcHeight = 0;
			var marginTop = 0;
			var footerHeight = $j('#workscape_footer').height();
			if(footerHeight === undefined){
				footerHeight = 0;
			}
			wcCalcHeight += footerHeight;
			if($M.isMobile()){
				wcCalcHeight += 50;
				marginTop = height - footerHeight;    
			} else {
				marginTop = height;   
			}
			var maxHeight = (fixed ? 'calc(100vh - '+(height + wcCalcHeight)+'px)' : this.maxHeight);
			tbody.css({'max-height': maxHeight});
			if(fixed && pos.width){
				tbody.css({'width':(pos.width - scrollBarWidth)+'px', 'margin-top': marginTop+'px'});
			} else {
				tbody.css({'width':'auto', 'margin-top': '0px'});
			}
	  	},
	  	
	  	_setNoRecordsLabel: function(customlabel){
	  		return (customlabel) ? customlabel : $M.localize('uitext','mxapibase','NoRecordsForTable');
	  	},
	  	
	  	_showSpinner:function(colnum, datacolumn){
	  		return (this.numericspinnercolnum!='' && colnum==this.numericspinnercolnum && datacolumn.numericspinnerattribute);
	  	},
	  	
	  	_showMaximoText: function(colnum) {
	  		return colnum.type && (colnum.type === 'maximotext');
	  	},
	  	
	  	_getSpinnerValue:function(record){
	  		if (this.actionCallback){
				return this.actionCallback.apply(this, [record]);
			}
	  	},
	  	
	  	/**
	  	 * update value - manual input for numeric spinner
	  	 */
	  	_updateValue:function(e){
	  		e.currentTarget._updateValue();
	  		// Event fire to handle manual input value for numeric spinner
	  		this.fire('maximo-table-rows-selected', this);
	  		this.fire('spinner-value-changed', {'record':this,'target':e.currentTarget,'rowrecord':e.model.record});
	  	},
	  	_onChangedValue:function(e){
	  		// Event fire to handle manual input value for numeric spinner
	  		this.fire('maximo-table-rows-selected', this);
	  		this.fire('spinner-value-changed', {'record':this,'target':e.currentTarget,'rowrecord':e.model.record});
	  	},
	  	_inputChanged:function(e){
	  		this.fire('maximo-table-rows-selected', this);
	  		this.fire('text-value-changed', {'record':this,'target':e.currentTarget});
	  	},
	  	/**
         * Get table column id 
         */
        _getID:function(column){
        	  return column.attribute?column.attribute:column.numericspinnerattribute;
         },
         /**
         * Display loader on collection refresh
         */
        _collectionRefreshRequested: function(e){
        	 this.items = [];
        	 this._loading = true;
         },
         
         /**
          * Function to scan barcode 
          */
        _returnQRcode : function(event){
	    	if(this.filterRow){
	    	 var self=this;
	         this._displayColumns.forEach(function(column, index){
	                 if(column.attribute && column.enablescan){
	                	 column.filterValue=event.detail.value.toUpperCase();
	                 }
	             });
	         this._applyFilter();
	         this._showFilterData(); 
	    	}
         },
         
        _showFilterData :function(self){
			var self=this;
			var filterId='';
			var filterData= this.collection.filterData[0];
			if(!this.filterRow && 
					filterData){
				this._toggleFilter();
			}
			if(filterData && filterData.availablevalues){
				var filterValue=filterData.availablevalues[0].value;				
				self._displayColumns.forEach(function(column){
					if(column.enablescan){
						filterId='#'+self.id+'_filter_input_'+column.attribute;
						return false;
					}
				});
				
			setTimeout(function() {
			   var filterRow = $j(self).find('#tableFilterRow');
			   if(filterRow){
				  $j(filterRow).find(filterId)[0].value = filterValue;
			   }
    	    }, 100);
			}
		},
		
		 _getIindex: function(index) {
	    	 return index + 1 + '.';
	     },
	     
	     /**
         Fires an event indicating that an header action was done. The parent element can handle it however they want.
	     **/
	     tableHeaderAction: function(e){
	         var toSend = {
	             action: e.model.headeraction,
	             source: e.currentTarget,
	         };
	         return this.fire('table-header-action-clicked',toSend);
	     },
	     _getLookupObject: function(record, datacolumn){
	    	 if(datacolumn.haslookup){
	    		 this.async(function(){
		    		 var source = $j('#input'+$M.makeSafeId(record._id)).closest('maximo-text')[0];
			    	 var toSend = {
			             record: record,
			             source: source,
			             datacolumn: datacolumn
				     };
			    	 if(datacolumn.lookup.customhref){
			    		 this.fire('lookup-object', toSend);
			    	 } else {
			    		 var lookupObj = {};
			    		 lookupObj['name'] = datacolumn.lookup.name;
			    		 lookupObj['href'] = record.href;
			    		 source.lookup = JSON.stringify(lookupObj);
			    	 }
		    	 }, 200);
	    	 } 
	    	 
	     },
	     
	     _toggleActions: function(actions, scanner, filter) {
	    	 var toggleAction = scanner || filter || this._hasActions(actions);
	    	 return toggleAction === undefined ? false: toggleAction;
	     }
});